#include "non_gui_defs.h"
#include "interface.h"
#include "support.h"
#include "callbacks.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "gui_defs.h"

/* these two functions are copied from the glade-generated
   support.c file to handle my pixmaps */
void
my_add_pixmap_directory                   (const gchar     *directory)
{
    my_pixmaps_directories = g_list_prepend (my_pixmaps_directories,
					     g_strdup (directory));
}

gchar*
my_find_pixmap_file                       (const gchar     *filename)
{
  GList *elem;

  /* We step through each of the pixmaps directory to find it. */
  elem = my_pixmaps_directories;
  while (elem)
    {
      gchar *pathname = g_strdup_printf ("%s%s%s", (gchar*)elem->data,
                                         G_DIR_SEPARATOR_S, filename);
      if (g_file_test (pathname, G_FILE_TEST_EXISTS))
        return pathname;
      g_free (pathname);
      elem = elem->next;
    }
  return NULL;
}

/* write a message into the message window */
gint print_message(GtkWidget *widget , gchar *message)
{
    GtkWidget *message_window = lookup_widget(widget, "message_window");
    const gchar *current_message;
    
    gtk_entry_set_text( GTK_ENTRY(message_window), message);

    if(options_misc[4] == 0)
	gtk_entry_prepend_text( GTK_ENTRY(message_window), "  ");
    
    options_misc[4] = (options_misc[4] + 1) % 2;
    
    return 0;
}

/* get an integer from a label */
gint get_int_from_label(GtkLabel *label)
{
    gchar *string;
    gint number;
    
    gtk_label_get(label, &string);
    
    number = (gint)strtol(string, NULL, 10);
    
    return number;
}

/* write new kennel id and owner */
gint write_dog_list_label(GtkWidget *widget, gint new_kennel_id)
{
    GtkWidget *current_kennel_owner = lookup_widget(widget,
						    "current_kennel_owner");
    GtkWidget *current_kennel_id = lookup_widget(widget, "current_kennel_id");
    GtkWidget *current_kennel_owner_name =
	lookup_widget(widget, "current_kennel_owner_name");

    gint i;

    /* write data */    
    label_set_text_from_int( GTK_LABEL(current_kennel_id), new_kennel_id, 0);

    if(new_kennel_id < 21)
    {
	gtk_label_set_text( GTK_LABEL(current_kennel_owner), "Owner: ");
	gtk_label_set_text( GTK_LABEL(current_kennel_owner_name),
			    kennels[new_kennel_id - 1].name);
    }
    else
    {
	gtk_label_set_text( GTK_LABEL(current_kennel_owner), "Your puppies ");
	gtk_label_set_text( GTK_LABEL(current_kennel_owner_name), "");
    }
    
    return 0;    
}

/* get order index of the currently selected dog */
gint get_selected_row(GtkWidget *widget, gint dog_info)
{
    GtkWidget *selected_row;
    gchar *sel_row_string;
    gint sel_row;
    
    if(dog_info == 0)
	selected_row = lookup_widget(widget, "selected_row_dog_list");
    else
	selected_row = lookup_widget(widget, "selected_row_dog_info");
    
    gtk_label_get( GTK_LABEL(selected_row), &sel_row_string );

    sel_row = (gint)strtol(sel_row_string, NULL, 10);
    
    return sel_row;
}

/* write id of the currently selected dog */
gint write_selected_row(GtkWidget *widget, gint new_sel_row, gint dog_info)
{
    GtkWidget *selected_row;
    
    if(dog_info == 0)
	selected_row = lookup_widget(widget, "selected_row_dog_list");
    else
	selected_row = lookup_widget(widget, "selected_row_dog_info");
    
    label_set_text_from_int( GTK_LABEL(selected_row), new_sel_row, 0);

    return 0;
}

/* print detailed information about a dog into the
   dog_info list in the main window;
   called when the human player selects a dog in the
   dog_list list */
gint print_dog_info(dog Dog, GtkCList *dog_info)
{
    gint i,j;
    gchar *row[8];
    gchar temporary_gchar[50];

    gtk_clist_freeze(GTK_CLIST(dog_info));
    gtk_clist_clear(GTK_CLIST(dog_info));

    for(i=2;i<8;i++)
	namescan(&row[i], "");

    namescan( &row[0], "Name:");
    namescan( &row[1], Dog.name);
    gtk_clist_append( dog_info, row); g_free(row[0]); g_free(row[1]);

    namescan( &row[0], "Age:");
    number_to_char( &row[1], Dog.age, 2);
    gtk_clist_append( dog_info, row); g_free(row[0]); g_free(row[1]);

    namescan( &row[0], "Gender:");
    if(Dog.gender == 0)
	namescan( &row[1], "Male" );
    else
	namescan( &row[1], "Female" );
    gtk_clist_append( dog_info, row); g_free(row[0]); g_free(row[1]);

    namescan( &row[0], "Level");
    number_to_char( &row[1], (gfloat) (Dog.level % 10), 1);
    gtk_clist_append( dog_info, row); g_free(row[0]); g_free(row[1]);


    namescan( &row[0], "Speed:");
    number_to_char( &row[1], real_value(Dog,1), 2);
    gtk_clist_append( dog_info, row); g_free(row[0]); g_free(row[1]);

    namescan( &row[0], "Stamina:");
    number_to_char( &row[1], real_value(Dog,0), 2);
    gtk_clist_append( dog_info, row); g_free(row[0]); g_free(row[1]);

    namescan( &row[0], "Points this season:");
    number_to_char( &row[1], (gfloat)points_of(Dog), 3);
    gtk_clist_append( dog_info, row); g_free(row[0]); g_free(row[1]);

    namescan( &row[0], "Status:");

    /* in heat */
    if( Dog.status == 0 && Dog.age > 11)
	strcpy(temporary_gchar, "in heat");
    /* injury */
    else if(19 < Dog.status && Dog.status < 28)
	sprintf(temporary_gchar, "injured, %d weeks", (Dog.status - 19) * 2);
    /* recovering from birth */
    else if(Dog.status == 50)
	strcpy(temporary_gchar, "recovering");
    else if(Dog.status > 1000 && Dog.status < 4000)
    {
	strcpy(temporary_gchar, "pregnant by ");
	strcat(temporary_gchar, dogs[ Dog.status % 1000 - 100 ].name);
    }
    else if(Dog.status > 4000)
	strcpy(temporary_gchar, "? pregnant ?");
    else
	strcpy(temporary_gchar, "OK");

    namescan( &row[1], temporary_gchar);
    
    gtk_clist_append( dog_info, row); g_free(row[0]); g_free(row[1]);

    /* now some info that's not visible in the normal dog
       list */
    namescan( &row[0], "");
    namescan( &row[1], "");
    gtk_clist_append( dog_info, row); g_free(row[0]); g_free(row[1]);

    namescan( &row[0], "Mother:");
    if(strcmp(Dog.mother_name, "NAME") != 0)
	namescan( &row[1], Dog.mother_name);
    else
	namescan( &row[1], "unknown");
    gtk_clist_append( dog_info, row); g_free(row[0]); g_free(row[1]);

    namescan( &row[0], "Father:");
    if(strcmp(Dog.father_name, "NAME") != 0)
	namescan( &row[1], Dog.father_name);
    else
	namescan( &row[1], "unknown");
    gtk_clist_append( dog_info, row); g_free(row[0]); g_free(row[1]);
    
    for(i=0;i<2;i++)
    {
	if(i == 0)
	    namescan( &row[0], "Best times this season:");
	else
	    namescan( &row[0], "Best times ever:");

	row[1] = (char*) xmalloc(19);
	strcpy(row[1], "\0");
	for(j=0;j<3;j++)
	    if(Dog.best_times[i * 3 + j] == 100)
		strcat(row[1], "--.-- ");
	    else
	    {
		sprintf( temporary_gchar, "%.2f ",Dog.best_times[i * 3 + j] );
		strcat( row[1], temporary_gchar);
	    }

	gtk_clist_append( dog_info, row ); g_free(row[0]); g_free(row[1]);
    }

    namescan( &row[0], "Races this season:");
    number_to_char( &row[1], (gfloat)Dog.races[0], 2);
    gtk_clist_append( dog_info, row ); g_free(row[0]); g_free(row[1]);

    namescan( &row[0], "Results (wins, 2nd - 5th places):");
    row[1] = (char*)xmalloc(15);
    strcpy(row[1], "\0");
    for(i=1;i<6;i++)
    {
	sprintf(temporary_gchar, "%d ", Dog.races[i]);
	strcat( row[1], temporary_gchar);
    }
    gtk_clist_append( dog_info, row ); g_free(row[0]); g_free(row[1]);

    namescan( &row[0], "Points per race:");
    if(Dog.races[0] > 0)
	number_to_char( &row[1],
			(gfloat)points_of(Dog) / (gfloat)Dog.races[0], 6 );
    else
	namescan( &row[1], "0");

    gtk_clist_append( dog_info, row); g_free(row[0]); g_free(row[1]);

    namescan( &row[0], "Winning rate:" );
    if(Dog.races[0] > 0)
    {
	sprintf(temporary_gchar, "%.1f%%",
		100 * (gfloat)Dog.races[1] / (gfloat)Dog.races[0]);
	namescan( &row[1], temporary_gchar);
    }
    else
	namescan( &row[1], "0,0%" );

    gtk_clist_append( dog_info, row ); g_free(row[0]); g_free(row[1]);

    namescan( &row[0], "Last 5 race finishes: ");
    row[1] = (char*)xmalloc(15);
    strcpy(row[1], "\0");

    for(i=0;i<5;i++)
    {
	if((gint)Dog.last_races[i*2] != 15)
	    sprintf(temporary_gchar, "%d ", (gint)Dog.last_races[i*2]);
	else
	    strcpy(temporary_gchar, "- ");

	strcat( row[1], temporary_gchar);
    }
    gtk_clist_append( dog_info, row ); g_free(row[0]); g_free(row[1]);

    /* has there been a level change? */
    if(	Dog.level > 200 )
    {
	namescan( &row[0], "Level change: ");
	sprintf(temporary_gchar, "%d --> %d", Dog.level % 10 - 1,
		Dog.level % 10);
	namescan( &row[1], temporary_gchar);
	gtk_clist_append( dog_info, row ); g_free(row[0]); g_free(row[1]);
	
	/* make it colorful */
	change_color_row( dog_info, clist_last_row(dog_info), 2);
    }
    else if( Dog.level > 99 && Dog.level < 110 )
    {
	namescan( &row[0], "Level change: ");
	sprintf(temporary_gchar, "%d --> %d", Dog.level % 10 + 1,
		Dog.level % 10);
	namescan( &row[1], temporary_gchar);
	gtk_clist_append( dog_info, row ); g_free(row[0]); g_free(row[1]);
	
	change_color_row( dog_info, clist_last_row(dog_info), 3);
    }

    gtk_clist_thaw(GTK_CLIST(dog_info));
    
    return 0;
}

/* change the title of the dog_info list.
   it's can be either:
   type = 0: the title of a detailed dog information window
   type = 10-29: a kennel
   type = 30: race results shown at the end of a semi-month
   type = 31: a race list of entered dogs
   type = 32: results of last races
   type = 40: immediate results of trophy races
   type = 41: entering dogs in trophy races
   type = 42: results of trophy races
   type = 50: dog or kennel statistics
   type = 60: other statistics */
gint write_dog_info_label(GtkWidget *widget, gint level_or_kennel_id,
			 gint laps, gint race_number, gint type)

{
    GtkWidget *dog_info_label;
    GtkWidget *dog_info_label_2;
    GtkWidget *dog_info_label_3;
    GtkWidget *dog_info_label_4;
    GtkWidget *dog_info_label_5;
    GtkWidget *dog_info_label_6;

    dog_info_label = lookup_widget(widget, "dog_info_label");
    dog_info_label_2 = lookup_widget(widget, "dog_info_label_2");
    dog_info_label_3 = lookup_widget(widget, "dog_info_label_3");
    dog_info_label_4 = lookup_widget(widget, "dog_info_label_4");
    dog_info_label_5 = lookup_widget(widget, "dog_info_label_5");
    dog_info_label_6 = lookup_widget(widget, "dog_info_label_6");

    gtk_label_set_text(GTK_LABEL(dog_info_label), "");
    gtk_label_set_text(GTK_LABEL(dog_info_label_2), "");
    gtk_label_set_text(GTK_LABEL(dog_info_label_3), "");
    gtk_label_set_text(GTK_LABEL(dog_info_label_4), "");
    gtk_label_set_text(GTK_LABEL(dog_info_label_5), "");
    gtk_label_set_text(GTK_LABEL(dog_info_label_6), "");

    if(type == 0)
	gtk_label_set_text(GTK_LABEL(dog_info_label),
			   " Detailed dog information");
    else if(type > 29 && type < 43)
    {
	if(type % 10 == 0)
	    gtk_label_set_text(GTK_LABEL(dog_info_label),
			       " Race results  Level: ");
	else if(type % 10 == 1)
	    gtk_label_set_text(GTK_LABEL(dog_info_label),
			       " Entering dogs  Level: ");
	else
	    gtk_label_set_text(GTK_LABEL(dog_info_label),
			       " Last races  Level: ");

	if(level_or_kennel_id != -1)
	    label_set_text_from_int(GTK_LABEL(dog_info_label_2), 
				    level_or_kennel_id, 0);
	else
	    gtk_label_set_text(GTK_LABEL(dog_info_label_2), "any");

	gtk_label_set_text(GTK_LABEL(dog_info_label_3), " Laps: ");
	
	label_set_text_from_int(GTK_LABEL(dog_info_label_4), 
				laps, 0);

	gtk_label_set_text(GTK_LABEL(dog_info_label_5), " Race: ");	

	label_set_text_from_int(GTK_LABEL(dog_info_label_6),
				race_number, 0);

    }
    else if(type > 9 && type < 30)
    {
	if(type == 20)
	    gtk_label_set_text(GTK_LABEL(dog_info_label),
			       " Trading with  Kennel: ");
	else
	    gtk_label_set_text(GTK_LABEL(dog_info_label),
			       " Mating dogs  Kennel: ");

	label_set_text_from_int(GTK_LABEL(dog_info_label_2), 
				level_or_kennel_id, 0);

	gtk_label_set_text(GTK_LABEL(dog_info_label_3), " Owner: ");
	gtk_label_set_text(GTK_LABEL(dog_info_label_4),
			   kennels[level_or_kennel_id - 1].name);
    }
    else if(type == 50)
    {
	if(level_or_kennel_id == 10)
	{
	    gtk_label_set_text(GTK_LABEL(dog_info_label),
			       " Statistics  Kennels ");
	    gtk_label_set_text(GTK_LABEL(dog_info_label_2), "");	    
	}
	else if(level_or_kennel_id == 20)
	{
	    gtk_label_set_text(GTK_LABEL(dog_info_label),
			       " Statistics  Dogs on level: ");
	    gtk_label_set_text(GTK_LABEL(dog_info_label_2), " all ");	    
	}
	else
	{
	    gtk_label_set_text(GTK_LABEL(dog_info_label),
			       " Statistics  Dogs on level: ");
	    label_set_text_from_int(GTK_LABEL(dog_info_label_2),
				    level_or_kennel_id, 0);
	}
	
	gtk_label_set_text(GTK_LABEL(dog_info_label_3), "Page: ");

	label_set_text_from_int(GTK_LABEL(dog_info_label_4), 
				laps, 0);
    }
    else if(type == 60 || type == 70)
    {
	gtk_label_set_text(GTK_LABEL(dog_info_label), " Statistics  ");

	if(type == 70)
	    gtk_label_set_text(GTK_LABEL(dog_info_label_2),
			       "All Time Records");
	else
	    gtk_label_set_text(GTK_LABEL(dog_info_label_2),
			       "Champions' Race ");
    }
    else if(type == 80)
    {
	gtk_label_set_text(GTK_LABEL(dog_info_label),
			   " Statistics  ");
	gtk_label_set_text(GTK_LABEL(dog_info_label_2), "History  Season ");
	label_set_text_from_int(GTK_LABEL(dog_info_label_3),
				level_or_kennel_id, 0);
    }


    return 0;
}

/* format 'dog_info' fittingly to display
   type = 0: detailed dog info
   type = 10 - 29: a cpu-kennel dog list
   type = 30: race results
   type = 31: race entries
   type = 32: last races
   type = 50: dog or kennel statistics
   type = 60: champions' race statistics
   type = 70: all-time records
   type = 80: history statistics */
gint format_dog_info(GtkWidget *widget, gint *width, gchar title[][16],
		    gint level_or_kennel_id, gint laps, gint race_number,
		    gint type)
{
    GtkWidget *dog_info = lookup_widget(widget, "dog_info");
    gint i;
    
    /* format dog_info window title */
    write_dog_info_label(widget, level_or_kennel_id,
			 laps, race_number, type);
    
    if(type < 30 && type > 9)
	justify_clist(GTK_CLIST(dog_info), 0, 1, 1, 1, 1, 1, 1, 1);
    else if(type > 29 && type < 43)
	justify_clist(GTK_CLIST(dog_info), 1, 0, 1, 0, 1, 1, 1, 1);
    else if(type == 50 || type == 60)
    {
	if(type == 50)
	    justify_clist(GTK_CLIST(dog_info), 0, 0, 1, 1, 1, 1, 1, 1);
	else
	    justify_clist(GTK_CLIST(dog_info), 1, 0, 1, 1, 1, 1, 1, 1);
    }
    else if(type == 70)
	justify_clist(GTK_CLIST(dog_info), 0, 1, 0, 1, 0, 0, 0, 0);
    else if(type == 80)
	justify_clist(GTK_CLIST(dog_info), 0, 0, 0, 1, 0, 0, 0, 0);
    else
	justify_clist(GTK_CLIST(dog_info), 0, 0, 0, 0, 0, 0, 0, 0);

    for(i=0;i<8;i++)
	gtk_clist_set_column_width( GTK_CLIST(dog_info), i, width[i] );
    for(i=0;i<8;i++)
	gtk_clist_set_column_title( GTK_CLIST(dog_info), i, title[i]);


    if(type != 0 && type != 80)
	gtk_clist_column_titles_show(GTK_CLIST(dog_info));
    else
	gtk_clist_column_titles_hide(GTK_CLIST(dog_info));

    return 0;
}

/* return the order index corresponding to the selected row */
gint get_order_index_from_row(GtkCList *clist, gint kennel_id, gint row)
{
    gint i;
    gint counter = row;
    gint order_index = 0;
    gchar *temporary_gchar = NULL;
    
    /* first try to get the index by skipping all dogs
       with age -1 */
    for(i = kennel_id * 20; i < (kennel_id + 1) * 20; i++)
    {
	if(dogs[order[i]].age != -1)
	{
	    counter--;
	    if(counter == -1)
		order_index = i;
	}
    }

    /* this doesn't work, though, when only dogs with a special
       attribute are displayed; e.g. when only males or females
       in heat are displayed. so now we check whether the name's
       right and look for the row by name otherwise */
    gtk_clist_get_text(clist, row, 0, &temporary_gchar);

    if(temporary_gchar == NULL)
	return get_order_index_from_row(clist, kennel_id, 0);

    if(strcmp(temporary_gchar, dogs[order[order_index]].name) == 0)
	return order_index;
    else
    {
	for(i = kennel_id * 20; i < (kennel_id + 1) * 20; i++)
	{
	    if(strcmp(temporary_gchar, dogs[order[i]].name) == 0)
		return i;
	}
    }

    return -1;
}

/* show either race results or entry lists in the dog_info window.
   type = 30: immediate results shown at the end of a semi-month 
   type = 31: entering dogs in races
   type = 32: results of last semi-month
   type = 40: immediate results of trophy races
   type = 41: entering dogs in trophy races
   type = 42: results of trophy races */
gint display_races(GtkWidget *widget, gint race_number, gint type)
{
    GtkWidget *dog_info = lookup_widget(widget, "dog_info");
    gint dog_info_width[8] = {20, 130, 40, 115, 100, 0, 0, 0};
    gchar dog_info_title[8][16];
    gint race_type = (type % 10 == 1) ? 1 : 0;
    gint real_race_number;
        
    /* find out the real race number, i.e., the
       real array index. */
    /* if we have trophy races here, there are only 6 races
       instead of 12 */
    if( type > 39 )
    {
	if(race_number == 0)
	    race_number = 6;
	else if(race_number == 7)
	    race_number = 1;
    }
    else
    {
	if(race_number == 0)
	    race_number = 12;
	else if(race_number == 13)
	    race_number = 1;
    }

    real_race_number = race_number - 1;
    
    strcpy(dog_info_title[0], "");    
    strcpy(dog_info_title[1], "Name");

    if(race_type == 1)
	strcpy(dog_info_title[2], "BTS");
    else
	strcpy(dog_info_title[2], "Time");
    
    strcpy(dog_info_title[3], "Kennel");
    if(type < 39)
	strcpy(dog_info_title[4], "Remarks");
    else
	strcpy(dog_info_title[4], "Points");
    strcpy(dog_info_title[5], "");
    strcpy(dog_info_title[6], "");
    strcpy(dog_info_title[7], "");
    
    /* change title of the dog_info list and format it
       fittingly */
    format_dog_info( widget, dog_info_width,
		     dog_info_title,
		     races[race_type][real_race_number].level,
		     races[race_type][real_race_number].laps,
		     race_number, type);

    gtk_clist_freeze(GTK_CLIST(dog_info));
    
    write_race_list(GTK_CLIST(dog_info), race_type,
		    real_race_number, type);
    
    /* show number of laps by color */
    set_laps_style(GTK_CLIST(dog_info),
		   races[race_type][real_race_number].laps);
    
    gtk_clist_thaw(GTK_CLIST(dog_info));

    return 0;
}

/* write a list of either a race result or of a race yet to come */
gint write_race_list(GtkCList *dog_info, gint race_type,
		    gint race_number, gint type)
{
    gint i;
    gint max_index = 
	(races[race_type][race_number].level == -1) ? 6 : 10;
    
    gtk_clist_clear(dog_info);

    for(i=0;i<max_index;i++)
	append_dog_to_race_list(dog_info, race_type,
				race_number, i, type);
    return 0;
}

/* 1st column: name 
   2nd: best time this season or race time
   3rd: dog's kennel
   4th: either remarks like 'best time ever', 'level up' etc., or,
   if it's a trophy race, points of each kennel */
gint append_dog_to_race_list(GtkCList *dog_info, gint race_type,
			    gint race_number, gint index, gint type)
{
    gint i, color = -1;
    gchar *row[8];
    gint idx = races[race_type][race_number].ranks[index];

    for(i=5;i<8;i++)
	namescan(&row[i], "");

    /* ranks */
    number_to_char( &row[0], index + 1, 2);

    if(races[race_type][race_number].dogs[idx] != NULL)
    {
	/* dog's name */
	namescan( &row[1], races[race_type][race_number].dogs[idx]->name );
	
	write_race_time(race_type, race_number, idx, &row[2]);

	write_race_remarks(race_type, race_number, idx, type, &color, &row[4]);

	/* kennel name */
	namescan( &row[3], kennels[ (gint)((gfloat)races[race_type][race_number].dogs[idx]->id / 20) ].name );
    }
    else
    {
	/* in trophy races we show all kennels and their points,
	   even if a dog hasn't been entered by the human player */
	if(races[race_type][race_number].level == -1)
	{
	    write_race_time(race_type, race_number, idx, &row[2]);
	    write_race_remarks(race_type, race_number, idx,
			       type, &color, &row[4]);
	    namescan( &row[3], kennels[0].name);

	    namescan(&row[1], "");
	}
	else
	    for(i=1;i<5;i++)
		namescan(&row[i], "");
    }	
    gtk_clist_append(dog_info, row);

    /* make it a bit colorful if there were special events
       concerning the human player's dogs or there's a new
       record  */
    if(races[race_type][race_number].dogs[idx] != NULL &&
       (races[race_type][race_number].dogs[idx]->id < 20 || color == 10))
    {
	if(color != -1)
	    change_color_row(dog_info, index, color);	
	else if(races[race_type][race_number].dogs[idx]->id < 20)
	    change_color_row(dog_info, index, 20);
    }
	
    for(i=0;i<8;i++)
	g_free(row[i]);
	
    return 0;
}

/* used in 'append_dog_to_race_list' */
gint write_race_time(gint race_type, gint race_number, gint idx, gchar **dest)
{
    gint laps = races[race_type][race_number].laps - 1;

    if(race_type == 1)
    {
	if( races[race_type][race_number].dogs[idx] != NULL &&
	    races[race_type][race_number].dogs[idx]->best_times[laps] == 100)
	    namescan( dest, "--:--");
	else if(races[race_type][race_number].dogs[idx] != NULL)
	    number_to_char( dest, races[race_type][race_number].
			    dogs[idx]->best_times[laps], 6);
	else
	    namescan( dest, "");
    }
    else
    {
	if(races[race_type][race_number].times[idx] > 99)
	    namescan( dest, "--:--");
	else
	    number_to_char( dest, races[race_type][race_number].times[idx], 6);
    }

    return 0;
}

/* write race remarks to 'dest' and note special events
   like level up etc. in '*color' */
gint write_race_remarks(gint race_type, gint race_number,
		       gint idx, gint type, gint *color, gchar **dest)
{
    gchar temporary_gchar[20] = "\0";
    gint dog_id = -1;
    gint kennel_id;
    gint laps = races[race_type][race_number].laps;

    /* find out whether the user's entered a dog */
    if(races[race_type][race_number].dogs[idx] == NULL)
	kennel_id = 0;
    else
    {
	dog_id = races[race_type][race_number].dogs[idx]->id;
	kennel_id = (gint)((gfloat)dog_id / 20);
    }

    /* no remarks when entering dogs */
    if(race_type == 1)
    {
	namescan( dest, "");
	return 1;
    }

    /* in trophy races we only write points and
       change color */
    if(type > 39)
	number_to_char( dest,
			trophy_points_of_kennel(kennel_id, race_number), 3);

    /* if there's no dog, we have to return here */
    if(races[race_type][race_number].dogs[idx] == NULL)
	return 2;

    if(races[race_type][race_number].times[idx] == 110)
    {
	if(type < 40)
	    strcat(temporary_gchar, "injury ");
	*color = 4;
    }
    /* is it best time ever? */
    else if(races[race_type][race_number].times[idx] ==
	    dogs[dog_id].best_times[laps + 2])
    {
	if(type < 40)
	    strcat(temporary_gchar, "BTE ");

	/* check for a new all-time record */
	if(races[race_type][race_number].times[idx] ==
	   dog_records[ races[race_type][race_number].laps - 1 ].value)
	{
	    *color = 10;
	    if(type < 40)
		strcat(temporary_gchar, "NR ");
	}
	else
	    *color = 0;
    }
    /* best time season? */
    else if(races[race_type][race_number].times[idx] ==
	    dogs[dog_id].best_times[laps - 1])
    {
	if(type < 40)
	    strcat(temporary_gchar, "BTS ");
	*color = 1;
    }
		
    /* level up? level down?
       level change coloring dominates best time 
       coloring and is on the other hand dominated
       by a record or an injury */
    if(dogs[dog_id].level > 99 &&
       dogs[dog_id].level < 110)
    {
	if(type < 40)
	    strcat(temporary_gchar, "LD");
	
	if(*color < 4)
	    *color = 3;
    }
    else if(dogs[dog_id].level > 200)
    {
	if(type < 40)
	    strcat(temporary_gchar, "LU");
	if(*color < 4)
	    *color = 2;
    }

    if(type < 40)
	namescan( dest, temporary_gchar);
    
    return 0;
}

/* if the user's entering dogs in races,
   we write additional information in shorter form
   into the message window */
gint print_short_dog_info(GtkWidget *widget, gint race_number, dog Dog)
{
    gint i;
    gint laps;
    gchar *temporary_gchar2;
    gchar temporary_gchar[20];
    GtkWidget *message_window = lookup_widget( widget, "message_window");

    /* get number of laps */
    laps = races[1][race_number].laps;

    /* best time for this distance in season */
    gtk_entry_set_text(GTK_ENTRY(message_window), "BTS: ");
    
    if(Dog.best_times[laps - 1] == 100)
    {
	gtk_entry_append_text(GTK_ENTRY(message_window), "--.-- ");
    }
    else
    {
	sprintf( temporary_gchar, "%.2f ",Dog.best_times[laps - 1] );
	gtk_entry_append_text(GTK_ENTRY(message_window), temporary_gchar);
    }

    
    gtk_entry_append_text(GTK_ENTRY(message_window), " Races: ");
    number_to_char( &temporary_gchar2, (gfloat)Dog.races[0], 2);
    gtk_entry_append_text(GTK_ENTRY(message_window), temporary_gchar2);
    g_free(temporary_gchar2);

    gtk_entry_append_text(GTK_ENTRY(message_window), " Results: ");

    for(i=1;i<6;i++)
    {
	sprintf(temporary_gchar, "%d ", Dog.races[i]);
	gtk_entry_append_text(GTK_ENTRY(message_window), temporary_gchar);
    }

    if(Dog.races[0] > 0)
    {
	gtk_entry_append_text(GTK_ENTRY(message_window), "Win %: " );
	number_to_char( &temporary_gchar2,
			(gfloat)Dog.races[1] / (gfloat)Dog.races[0], 6);
	gtk_entry_append_text(GTK_ENTRY(message_window), temporary_gchar2);
	g_free(temporary_gchar2);

	gtk_entry_append_text(GTK_ENTRY(message_window), " Pts/Race: ");
	number_to_char( &temporary_gchar2,
			(gfloat)points_of(Dog) / (gfloat)Dog.races[0], 6);
	gtk_entry_append_text(GTK_ENTRY(message_window), temporary_gchar2);
	g_free(temporary_gchar2);
    }
    else
    {
	gtk_entry_append_text(GTK_ENTRY(message_window), "Win %: 0");
	gtk_entry_append_text(GTK_ENTRY(message_window), " Pts/Race: 0");
    }
    
    return 0;
}

/* things that happen between race weekends */
gint between_race_weekends(GtkWidget *widget)
{
    gint i,j;
    gint temporary_gint;
    gint pup_info[4] = {0, 0, 0, 0};
    gint dog_info[3] = {0, 0, 0};
    GtkWidget *season_number = lookup_widget(widget, "season_number");
    GtkWidget *month_number = lookup_widget(widget, "month_number");
    GtkWidget *week_number = lookup_widget(widget, "week_number");
    GtkWidget *puppy_info = lookup_widget(widget, "puppy_info");
    GtkWidget *dogs_info = lookup_widget(widget, "dogs_info");
    GtkWidget *misc_events = lookup_widget(widget, "misc_events");
    
    /* clear message window */
    print_message(widget, "");
    
    /* set save status to 'unsaved' */
    set_save_status(widget, 0);

    /* increment month, week and season numbers */
    week = (week + 2) % 4;
    if(week == 1)
	month++;
    
    if(month == 13)
    {
	season++;
	month = 1;
    }
    
    label_set_text_from_int(GTK_LABEL(month_number), month, 0);
    label_set_text_from_int(GTK_LABEL(week_number), week, 0);
    label_set_text_from_int(GTK_LABEL(season_number), season, 0);
    
    /* if the previous was the last week of
       the season, tidy up things; nullify points etc. */
    if(month == 1 && week == 1 && season > 1)
    {
	season_ending(widget);
	/* this will cause the news with the 
	   trophy winner to pop up */
	gtk_label_set_text(GTK_LABEL(misc_events), "1");
    }

    /* check whether the all-time records have to be changed */
    check_records();

    /* dogs get older etc. */
    for(i=0;i<420;i++)
	if(dogs[i].age > -1)
	    {
		temporary_gint = semi_monthly_reevaluation(&dogs[i]);

		/* remove old dogs automatically */
		if(i < 20)
		{
		    if(dogs[i].age > 65)
		    {
			dog_info[0] = 1;
			dogs[i].age = -1;
		    }
		}
		/* find out if there's a bitch in heat
		   that's interesting for the human player.
		   dog_info[2] / 10 is the kennel number */
		else
		{
		    /* first condition: in heat */
		    if(dogs[i].status == 0)
		    {
			/* first case: the conjunction option
			   has to be evaluated */
			if(options_misc[18] < 100 && options_misc[19] < 100)
			{
			    /* OR */
			    if(options_misc[17] == 1 &&
			       (real_value(dogs[i],1) > options_misc[18] ||
				real_value(dogs[i],0) > options_misc[19]) )
				dog_info[2] =
				    (gint)((gfloat)dogs[i].id / 20) * 10;
			    /* AND */
			    else if(options_misc[17] == 0 &&
				    (real_value(dogs[i],1) > options_misc[18] &&
				     real_value(dogs[i],0) > options_misc[19]) )
				dog_info[2] =
				    (gint)((gfloat)dogs[i].id / 20) * 10;
			}
		    /* second case: the conjunction is not evaluated
		       because one limit is switched off */
			else
			    if(real_value(dogs[i],1) > options_misc[18] ||
			       real_value(dogs[i],0) > options_misc[19])
				dog_info[2] =
				    (gint)((gfloat)dogs[i].id / 20) * 10;
		    }
		}
		
		/* there was a birth */
		if( temporary_gint == 1)
		    pup_info[0] = 1;
		/* there was no room for all new-born
		   puppies */
		else if( temporary_gint == 2 )
		    pup_info[1] = 2;
		
		/* there are puppies that have
		   to be promoted */
		if(i > 399)
		{
		    /* promote automatically or remove */
		    if(dogs[i].age > 13)
		    {
			if(dogs_in_kennel(0) < 20)
			{
			    copy_dogs(dogs[i],
				      &dogs[first_free_place_in_kennel(0)]);
			    dogs[i].age = -1;
			}
			else
			{
			    dogs[i].age = -1;
			    pup_info[2] = 8;
			}
		    }
		    /* give a warning for the user:
		       he should promote some pups */
		    else if(dogs[i].age > 12.5)
			pup_info[3] = 4;
		}
	    }
    
    /* see if there was fighting leading to injuries
       in the kennel */
    dog_info[1] = fighting();
    
    /* when there's a birth we note it in the
       puppy_info label in the main window; when there wasn't
       enough room on the puppy list to place all new pups, dito.
       if there are puppies that should be promoted to dogs, we
       show a warning */
    label_set_text_from_int(GTK_LABEL(puppy_info),
			    pup_info[0] + pup_info[1] +
			    pup_info[2] +pup_info[3], 0);

    label_set_text_from_int(GTK_LABEL(dogs_info),
		   dog_info[0] + dog_info[1] +
		   dog_info[2], 0);    
    
    /* cpu-kennel owners forget (partly) the human player's
       stupid trading and mating offers of the past */
    for(j=0;j<2;j++)
	for(i=0;i<19;i++)
	    if(options_kennel_block[j][i] < 4)
		options_kennel_block[j][i] = 0;
	    else
		options_kennel_block[j][i] -= 3;

    /* compute new sorting order for the statistics windows.
       the user can change the order by simply clicking on the columns */
    sort_dogs_statistics(20, statistics_order[0]);
    sort_dogs_statistics(90, statistics_order[1]);
    sort_kennel_statistics(20, statistics_order[0], 400);
    sort_kennel_statistics(90, statistics_order[1], 400);
 
    /* compute new qualified list for the champions'
       trophy */
    write_qualified_kennels();

    /* if the season is over except the trophy races and
       the human player's first in one of the kennel categories,
       note it in the two 'selected_row' labels */
    if(month == 12 && week == 3)
    {
	write_new_history(0);
	awards_news(widget);
    }
    
    return 0;
}

/* nullify statistics and points etc. */
gint season_ending(GtkWidget *widget)
{
    gint i,j;

    /* write a new history struct */
    write_new_history(1);
    
    /* dogs: nullify number of races etc.
       and best times season */
    for(i=0;i<400;i++)
    {
	for(j=0;j<6;j++)
	    dogs[i].races[j] = 0;
	for(j=0;j<3;j++)
	    dogs[i].best_times[j] = 100;
    }

    /* cpu-owners forget the stupid offers the
       human player has made */
    for(i=0;i<2;i++)
	for(j=0;j<19;j++)
	    options_kennel_block[i][j] = 0;

    return 0;
}

/* set the colors of clist rows containing dog information
   (dogs on dog_list or dog_info) */
gint change_colors_clist(GtkCList *clist, gint dog_info)
{
    GtkWidget *selected_row;
    gint i;
    gint row; 
    gchar *temporary_gchar;

    if(dog_info == 0)
    {
	selected_row = lookup_widget(GTK_WIDGET(clist),
				     "selected_row_dog_list");
	row = get_int_from_label(GTK_LABEL(selected_row));
    }
    else
    {
	selected_row = lookup_widget(GTK_WIDGET(clist),
				     "selected_row_dog_info");
	row = get_int_from_label(GTK_LABEL(selected_row));
    }

    for(i=0;i<20;i++)
    {
	gtk_clist_get_text(clist, i, 7, &temporary_gchar);
	
	if(strcmp(temporary_gchar, "injured") == 0 ||
		strcmp(temporary_gchar, "recover.") == 0)
	{
	    if(row != i)
		change_color_row(clist, i, 21);
	    else
		change_color_row(clist, i, 4);
	}
	else if(strcmp(temporary_gchar, "in heat") == 0)
	{
	    if(row != i)
		change_color_row(clist, i, 22);
	    else
		change_color_row(clist, i, 10);
	}
	else if(strcmp(temporary_gchar, "pregnant") == 0)
	{
	    if(row != i)
		change_color_row(clist, i, 23);
	    else
		change_color_row(clist, i, 30);
	}
	else
	{
	    if(row != i)
		change_color_row(clist, i, 50);
	    else
		change_color_row(clist, i, 20);
	}
    }

    return 0;
}

/* set color for a clist row
   type = 0: BTE
   type = 1: BTS 
   type = 2: level up
   type = 3: level down
   type = 4: injury
   type = 10: new record or in heat
   type = 20: select row normal (white on dark blue)
   type = 21: select row injury (white on not as dark blue)
   type = 22: select row in heat (white on purple)
   type = 23: select row pregnant (white on dark green)
   type = 30: pregnant bitch normal
   type = 50: boring old black on white */
gint change_color_row(GtkCList *clist, gint row, gint type)
{
    GdkColor background_color, foreground_color;
    GdkColormap *cmap = gdk_colormap_get_system();

    /* the letters white or black */
    if(type < 2 || type == 4 || type == 50)
	foreground_color.red = foreground_color.green = 
	    foreground_color.blue = (guint16)0.0;	
    else
	foreground_color.red = foreground_color.green = 
	    foreground_color.blue = (guint16)65535.0;

    /* set background color */
    set_color(&background_color, type);

    gdk_colormap_alloc_color(cmap, &foreground_color, FALSE, TRUE);
    
    gtk_clist_set_foreground(clist, row, &foreground_color);
    gtk_clist_set_background(clist, row, &background_color);

    return 0;
}

gint set_color(GdkColor *color, gint type)
{
    GdkColormap *cmap = gdk_colormap_get_system();

    /* dog has reached best time ever */
    if(type == 0)
    {
	color->red = (guint16)1310.7;
	color->green = (guint16)44563.8;
	color->blue = (guint16)65535;
    }
    /* best time in current season */
    else if(type == 1)
    {
	color->red = (guint16)65535;
	color->green = (guint16)35388.9;
	color->blue = (guint16)0;
    }
    /* level up */
    else if(type == 2)
    {
	color->red = (guint16)1310.7;
	color->green = (guint16)32767.5;
	color->blue = (guint16)3276.75;
    }
    /* level down */
    else if(type == 3)
    {
	color->red = (guint16)37354.95;
	color->green = (guint16)5898.15;
	color->blue = (guint16)1966.05;
    }
    /* injury */
    else if(type == 4)
    {
	color->red = (guint16)0.0;
	color->green = (guint16)65535.0;
	color->blue = (guint16)65535.0;
    }
    /* new record */
    else if(type == 10)
    {
	color->red = (guint16)65535.0;
	color->green = (guint16)10486.0;
	color->blue = (guint16)0.0;	
    }
    else if(type == 20)
    {
	color->red = (guint16)(65535.0 * 0.15);
	color->green = (guint16)(65535.0 * 0.09);
	color->blue = (guint16)(65535.0 * 0.57);
    }
    else if(type == 21)
    {
	color->red = (guint16)(65535.0 * 0.22);
	color->green = (guint16)(65535.0 * 0.73);
	color->blue = (guint16)(65535.0 * 0.70);
    }
    else if(type == 22)
    {
	color->red = (guint16)(65535.0 * 0.80);
	color->green = (guint16)(65535.0 * 0.33);
	color->blue = (guint16)(65535.0 * 0.29);
    }
    else if(type == 23)
    {
	color->red = (guint16)(65535.0 * 0.46);
	color->green = (guint16)(65535.0 * 0.71);
	color->blue = (guint16)(65535.0 * 0.39);
    }
    else if(type == 30)
    {
	color->red = (guint16)(65535.0 * 0.03);
	color->green = (guint16)(65535.0 * 0.77);
	color->blue = (guint16)(65535.0 * 0.0);
    }
    else if(type == 50)
	color->red = color->green = color->blue = (guint16)65535.0;
    
    gdk_colormap_alloc_color(cmap, color, FALSE, TRUE);

    return 0;
}

/* mark first row of a race list with color depending
   on the number of laps so that the user doesn't have to
   decipher the race_laps_number label */
gint set_laps_style(GtkCList *dog_info, gint number_of_laps)
{
    gchar *row[8] = {"", "", "", "", "", "", "", ""};
    GdkColor background_color, foreground_color;
    GdkColormap *cmap = gdk_colormap_get_system();

    foreground_color.red = foreground_color.green = 
	foreground_color.blue = (guint16)0.0;

    if(number_of_laps == 1)
    {
	background_color.red = (guint16)(65535.0 * 0.93);
	background_color.green = (guint16)0.0;
	background_color.blue = (guint16)(65535.0 * 0.04);
    }
    else if(number_of_laps == 2)
    {
	background_color.red = (guint16)(65535.0 * 0.03);
	background_color.green = (guint16)(65535.0 * 0.53);
	background_color.blue = (guint16)(65535.0 * 0.84);
    }
    else if(number_of_laps == 3)
    {
	background_color.red = (guint16)0.0;
	background_color.green = (guint16)(65535.0 * 0.84);
	background_color.blue = (guint16)(65535.0 * 0.25);
    }
    else g_print("set_laps_style error %d\n", number_of_laps);

    gdk_colormap_alloc_color(cmap, &background_color, FALSE, TRUE);
    gdk_colormap_alloc_color(cmap, &foreground_color, FALSE, TRUE);
    
    gtk_clist_prepend(dog_info, row);
    
    gtk_clist_set_foreground(dog_info, 0, &foreground_color);
    gtk_clist_set_background(dog_info, 0, &background_color);
    
     
    return 0;
}

/* write a dog list of a kennel into a window.
   if doginfo is 0, the user is simply browsing
   through the kennels in dog_list.
   otherwise he's browsing through the cpu-kennels
   for trading or breeding.
   type = 0 just browsing kennels, looking at other's dogs
   type = 10 breeding; show all dogs  
   type = 11 breeding; show males
   type = 12 breeding; show females in heat
   type = 20 trading */ 
gint display_kennel(GtkWidget *widget, gint new_kennel_id,
		    gint doginfo, gint type)
{    
    GtkWidget *dog_list = lookup_widget(widget, "dog_list");
    GtkWidget *dog_info = lookup_widget(widget, "dog_info");
    GtkWidget *selected_row;
    gint row;
    gint rows_written;
    gchar *temporary_gchar = NULL;
    gint dog_info_width[8] = {130, 25, 22, 22, 25, 25, 30, 60};
    gchar dog_info_title[8][16];
    
    strcpy(dog_info_title[0], "Name");    
    strcpy(dog_info_title[1], "A");
    strcpy(dog_info_title[2], "G");
    strcpy(dog_info_title[3], "L");
    strcpy(dog_info_title[4], "SP");
    strcpy(dog_info_title[5], "ST");
    strcpy(dog_info_title[6], "Pts");
    strcpy(dog_info_title[7], "Status");

    /* set cycling mode: if trading, we skip the human player's kennel,
       and the puppies; if mating, we skip only the puppies */
    if(new_kennel_id == 22)
	new_kennel_id = 1;
    else if(new_kennel_id == 21)
    {
	if(type == 20)
	    new_kennel_id = 2;
	else if(type > 9 && type < 13)
	    new_kennel_id = 1;
    }
    else if(new_kennel_id == 0)
    {
	if(type != 0)
	    new_kennel_id = 20;
	else
	    new_kennel_id = 21;
    }
    else if(new_kennel_id == 1 && type == 20)
	new_kennel_id = 20;
    
    if(doginfo == 0 && type > 0)
	new_kennel_id = 1;

    /* find out which rows were selected to
       re-select them afterwards */
    if(doginfo == 0)
	selected_row =
	    lookup_widget(widget, "selected_row_dog_list");
    else
	selected_row =
	    lookup_widget(widget, "selected_row_dog_info");

    row = get_int_from_label(GTK_LABEL(selected_row));

    if(row > dogs_in_kennel(new_kennel_id - 1) - 1)
	row = 0;    

    /* freeze windows */
    gtk_clist_freeze(GTK_CLIST(dog_list));
    gtk_clist_freeze(GTK_CLIST(dog_info));

    /* write the labels and select the first dog */
    if(doginfo == 0)
    {
	gtk_clist_clear(GTK_CLIST(dog_list));
	justify_clist(GTK_CLIST(dog_list), 0, 1, 1, 1, 1, 1, 1, 1);

	rows_written = write_dog_list(widget, new_kennel_id - 1, 0, type);
	
	write_dog_list_label(widget, new_kennel_id);
	
	if(type == 0)
	{
	    dog_info_width[0] = 190;
	    dog_info_width[1] = 150;
	    
	    format_dog_info(widget, dog_info_width, dog_info_title,
			    new_kennel_id, 0, 0, 0);
	}	
	
	/* select the previously selected dog if possible */
	if(rows_written > row)
	    on_dog_list_select_row(GTK_CLIST(dog_list), row, 0, NULL,
				   NULL);
	else if(rows_written > 0)
	    on_dog_list_select_row(GTK_CLIST(dog_list), 0, 0, NULL,
				   NULL);

	/* clear dog_info if the dog_list is empty when in browsing mode */
	if(type == 0 && rows_written == 0)
	    gtk_clist_clear(GTK_CLIST(dog_info));
    }
    else
    {
	gtk_clist_clear(GTK_CLIST(dog_info));

	format_dog_info(widget, dog_info_width, dog_info_title,
			new_kennel_id, 0, 0, type);

	rows_written = write_dog_list(widget, new_kennel_id - 1, 1, type);

	if(rows_written > row)
	    on_dog_info_select_row(GTK_CLIST(dog_info), row, 0, NULL, NULL);
	else if(rows_written > 0)
	    on_dog_info_select_row(GTK_CLIST(dog_info), 0, 0, NULL, NULL);
    }

    /* thaw windows */
    gtk_clist_thaw(GTK_CLIST(dog_list));
    gtk_clist_thaw(GTK_CLIST(dog_info));

    return 0;
}

/* fill in a list with the dogs of a given kennel
   in a given order; either the dog_list window (dog_info = 0)
   or the dog_info window (dog_info = 1);
   type = 0, 1, 20: show all dogs 
   type = 10: show all healthy dogs (when interbreeding)
   type = 11: show only males (when interbreeding)
   type = 12: show only females in heat (when interbreeding) */
gint write_dog_list(GtkWidget *widget, gint kennel_id, gint dog_info,
		   gint type)
{
    GtkWidget *clist = (dog_info == 0) ?
	lookup_widget(widget, "dog_list") : lookup_widget(widget, "dog_info");
    gint i;
    gint rows_written = 0;
    
    for(i = kennel_id * 20; i < (kennel_id + 1) * 20; i++)
	if( dogs[ order[i] ].age > -1 &&
	    ( (type != 11 && type != 12) ||
	      (type == 11 && dogs[order[i]].gender == 0 &&
	       dogs[order[i]].status < 20) ||
	      (type == 12 && dogs[order[i]].status == 0)) )
	{
	    append_dog_to_dog_list(dogs[ order[i] ], GTK_CLIST(clist));
	    rows_written++;
	}
    
    return rows_written;
}

/* enter informations about a dog into a 'clist' */
gint append_dog_to_dog_list(dog Dog, GtkCList *dog_list)
{
    gchar *row[8];
    gchar temporary_gchar[50];
    
    namescan( &row[0], Dog.name); 

    sprintf( temporary_gchar, "%.0f", Dog.age);
    namescan( &row[1], temporary_gchar);

    if(Dog.gender == 0)
	strcpy( temporary_gchar, "M" );
    else
	strcpy( temporary_gchar, "F" );
    namescan( &row[2], temporary_gchar);

    sprintf( temporary_gchar, "%2d", Dog.level % 10);
    namescan( &row[3], temporary_gchar);
    
    sprintf( temporary_gchar, "%.0f", 
	     real_value(Dog,1) );
    namescan( &row[4], temporary_gchar);

    sprintf( temporary_gchar, "%.0f",
	     real_value(Dog,0) );
    namescan( &row[5], temporary_gchar);

    sprintf( temporary_gchar, "%3d",
	     points_of(Dog) );
    namescan( &row[6], temporary_gchar);  

    /* in heat; only for mature bitches */
    if( Dog.status == 0 && Dog.age > 11)
	strcpy(temporary_gchar, "in heat");
    /* injured */
    else if(19 < Dog.status && Dog.status < 28)
	strcpy(temporary_gchar, "injured");
    /* recovering from birth */
    else if(Dog.status == 50)
	strcpy(temporary_gchar, "recover.");
    else if( (Dog.status > 1000 && Dog.status < 4000) ||
	     (Dog.status > 5000 && Dog.status < 9000) )
	strcpy(temporary_gchar, "pregnant");
    else if( (Dog.status > 4000 && Dog.status < 5000) ||
	     Dog.status > 9000 )
	strcpy(temporary_gchar, "? pregnant ?");
    else
	strcpy(temporary_gchar, "OK");

    namescan(&row[7], temporary_gchar);

    gtk_clist_append( GTK_CLIST (dog_list), row);

    return 0;
}

/* remove selected dog from the user's kennel */
gint remove_dog(GtkWidget *widget)
{
    gint dog_indices[2];

    get_dog_indices(widget, dog_indices);

    dogs[dog_indices[0]].age = -1;

    return 0;
}

gint change_widget_color(GtkWidget *widget, gint type)
{
    GdkColor red = { 0, 65535.0 * 0.93, 0, 65535.0 * 0.04 };
    GtkRcStyle *rc_style = gtk_rc_style_new ();

    rc_style->fg[GTK_STATE_NORMAL] = red;
    rc_style->color_flags[GTK_STATE_NORMAL] |= GTK_RC_FG;
    gtk_widget_modify_style (widget, rc_style);
    gtk_rc_style_unref (rc_style);

    return 0;
}

/* return TRUE if selected dogs can (technichally) be mated,
   FALSE otherwise */
gboolean set_sensitive_mate_dogs(GtkWidget *widget)
{
    gint dog_indices[2];

    get_dog_indices(widget, dog_indices);

    if(dog_indices[0] == -1 ||
       dog_indices[1] == -1)
	return FALSE;

    /* opposite genders */
    if( dogs[dog_indices[0]].gender !=
	dogs[dog_indices[1]].gender &&
	/* bitch's in heat and dog's healthy */
	( (dogs[dog_indices[0]].status == 0 &&
	   dogs[dog_indices[1]].status < 20) ||
	  (dogs[dog_indices[0]].status < 20 &&
	   dogs[dog_indices[1]].status == 0) ) )
	return TRUE;
    
    return FALSE;    
}

/* find out the indices of the selected dogs on the
   two lists dog_list and dog_info */
gint get_dog_indices(GtkWidget *widget, gint *indices)
{
    GtkWidget *dog_list = lookup_widget(widget, "dog_list");
    GtkWidget *dog_info = lookup_widget(widget, "dog_info");
    GtkWidget *current_kennel_id = lookup_widget(widget,
						 "current_kennel_id");
    GtkWidget *dog_info_label = lookup_widget(widget,
					      "dog_info_label");
    GtkWidget *cpu_kennel = lookup_widget(widget,
					  "dog_info_label_2");
    gchar *temporary_gchar = NULL;
    gint cur_kennel_id = get_int_from_label(GTK_LABEL(current_kennel_id)) - 1;
    gint cpu_kennel_id = get_int_from_label(GTK_LABEL(cpu_kennel)) - 1;
    gint selected_row_dog_list = get_selected_row(widget, 0);
    gint selected_row_dog_info = get_selected_row(widget, 1);
    gint dog_list_index = -1;
	
    gint dog_info_index = -1;

    /* check whether there are dogs on the dog list */
    gtk_clist_get_text(GTK_CLIST(dog_list), 0, 0, &temporary_gchar);

    if(temporary_gchar != NULL)
	dog_list_index =
	    order[ get_order_index_from_row(GTK_CLIST(dog_list),
					    cur_kennel_id,
					    selected_row_dog_list) ];

    /* if there's a kennel in the dog_info window, look up
       the index */
    gtk_label_get(GTK_LABEL(dog_info_label), &temporary_gchar);
    
    if(strcmp(temporary_gchar, " Trading with  Kennel: ") == 0 ||
       strcmp(temporary_gchar, " Mating dogs  Kennel: ") == 0)
	dog_info_index = order[
	    get_order_index_from_row(GTK_CLIST(dog_info),
				     cpu_kennel_id, selected_row_dog_info) ];

    indices[0] = dog_list_index;
    indices[1] = dog_info_index;
    
    return 0;
}

/* print some news in a pop-up window */
gint print_news(GtkWidget *widget, gchar *news_text, gint type)
{
    GtkWidget *main_window = lookup_widget(widget, "main_window");
    /* this widget stores the number of active (not yet closed)
       pupup windows. we use it in callbacks to determine when to
       set the main window back to sensitive */
    GtkWidget *news_window = create_news_window();
    GtkWidget *news_window_text = lookup_widget(news_window,
						"news_window_text");
    GtkWidget *news_window_type = lookup_widget(news_window,
						"news_window_type");
    GtkWidget *news_window_checkbutton = 
	lookup_widget(news_window, "news_window_checkbutton");
    GtkWidget *news_window_image = 
	lookup_widget(GTK_WIDGET(news_window),
		      "news_window_image");
    
    gtk_label_set_line_wrap(GTK_LABEL(news_window_text), TRUE);
    gtk_label_set_text(GTK_LABEL(news_window_text), news_text);

    /* sometimes the checkbutton is not needed */
    if(type > 49)
	gtk_widget_hide(news_window_checkbutton);

    gtk_widget_show(news_window);
        
    if(type == 0)
	gtk_image_set_from_file(GTK_IMAGE(news_window_image),
				my_find_pixmap_file(
				    "main_buttons_breeding.xpm"));
    else if(type == 1 || type == 3 || type == 4)
	gtk_image_set_from_file(GTK_IMAGE(news_window_image),
				my_find_pixmap_file(		 
				    "dog_buttons_give_away.xpm"));
    else if(type == 5)
	gtk_image_set_from_file(GTK_IMAGE(news_window_image),
				my_find_pixmap_file(		 
				    "events_injury.xpm"));
    else if(type == 6)
	gtk_image_set_from_file(GTK_IMAGE(news_window_image),
				my_find_pixmap_file(		 
				    "events_trophy.xpm"));
    else if(type == 7)
	gtk_image_set_from_file(GTK_IMAGE(news_window_image),
				my_find_pixmap_file(		 
				    "events_in_heat.xpm"));
    else if(type == 60)
	gtk_image_set_from_file(GTK_IMAGE(news_window_image),
				my_find_pixmap_file(		 
				    "events_champion.xpm"));
    else
	gtk_image_set_from_file(GTK_IMAGE(news_window_image),
				my_find_pixmap_file("events_warning.png"));
    
    /* increase 'popups_active' and set main window
       non-sensitive */
    change_popups_active(widget, 1);

    /* set news_window type */
    label_set_text_from_int(GTK_LABEL(news_window_type), type, 0);
        
    return 0;
}

/* set all buttons insensitive, so that other functions
   only have to look after setting some to sensitive */
gint set_buttons_non_sensitive(GtkWidget *widget)
{
    GtkWidget *toolbar_quit = lookup_widget(widget,
					    "toolbar_quit");
    GtkWidget *toolbar_load = lookup_widget(widget,
					    "toolbar_load");
    GtkWidget *toolbar_save = lookup_widget(widget,
					    "toolbar_save");
    GtkWidget *toolbar_options = lookup_widget(widget,
					    "toolbar_options");
    GtkWidget *toolbar_home = lookup_widget(widget,
					    "toolbar_home");    
    GtkWidget *toolbar_back = lookup_widget(widget,
					    "toolbar_back");
    GtkWidget *toolbar_forward = lookup_widget(widget,
					    "toolbar_forward");
    GtkWidget *back_to_main = lookup_widget(widget,
					    "main_buttons_back_to_main");
    GtkWidget *enter_dogs_in_races = lookup_widget(widget,
						   "main_buttons_enter_dogs");
    GtkWidget *go_to_races = lookup_widget(widget,
					   "main_buttons_race");
    GtkWidget *show_race_results = lookup_widget(widget,
						 "main_buttons_results");
    GtkWidget *trading = lookup_widget(widget,
				       "main_buttons_trading");
    GtkWidget *breeding = lookup_widget(widget,
				       "main_buttons_breeding");
    GtkWidget *show_puppies = lookup_widget(widget,
					    "show_puppies");
    GtkWidget *enter_dog_in_race = lookup_widget(widget,
						 "dog_buttons_enter_dog");
    GtkWidget *remove_dog_from_race = lookup_widget(widget,
						    "dog_buttons_remove_dog");
    GtkWidget *trade_dogs = lookup_widget(widget,
					  "dog_buttons_trade_dogs");
    GtkWidget *mate_dogs = lookup_widget(widget,
					 "dog_buttons_mate_dogs");
    GtkWidget *promote_pup = lookup_widget(widget,
					   "dog_buttons_promote_pup");
    GtkWidget *give_away_dog = lookup_widget(widget,
					     "dog_buttons_give_away");
    GtkWidget *statistics = lookup_widget(widget,
					  "main_buttons_statistics");        
    
    gtk_widget_set_sensitive(toolbar_quit, FALSE);
    gtk_widget_set_sensitive(toolbar_load, FALSE);
    gtk_widget_set_sensitive(toolbar_save, FALSE);
    gtk_widget_set_sensitive(toolbar_options, FALSE);
    gtk_widget_set_sensitive(toolbar_home, FALSE);
    gtk_widget_set_sensitive(toolbar_back, TRUE);
    gtk_widget_set_sensitive(toolbar_forward, TRUE);
    gtk_widget_set_sensitive(back_to_main, FALSE);
    gtk_widget_set_sensitive(enter_dogs_in_races, FALSE);
    gtk_widget_set_sensitive(go_to_races, FALSE);
    gtk_widget_set_sensitive(show_race_results, FALSE);
    gtk_widget_set_sensitive(trading, FALSE);
    gtk_widget_set_sensitive(breeding, FALSE);
    gtk_widget_set_sensitive(show_puppies, FALSE);
    gtk_widget_set_sensitive(enter_dog_in_race, FALSE);
    gtk_widget_set_sensitive(remove_dog_from_race, FALSE);
    gtk_widget_set_sensitive(trade_dogs, FALSE);
    gtk_widget_set_sensitive(mate_dogs, FALSE);
    gtk_widget_set_sensitive(promote_pup, FALSE);
    gtk_widget_set_sensitive(give_away_dog, FALSE);
    gtk_widget_set_sensitive(statistics, FALSE);
    
    return 0;
}

/* set the sensitivity of buttons depending on the menu the user's
   in and the selected rows on the two lists */
gint set_buttons(GtkWidget *widget)
{
    GtkWidget *toolbar_quit = lookup_widget(widget,
					    "toolbar_quit");
    GtkWidget *toolbar_load = lookup_widget(widget,
					    "toolbar_load");
    GtkWidget *toolbar_save = lookup_widget(widget,
					    "toolbar_save");
    GtkWidget *toolbar_options = lookup_widget(widget,
					       "toolbar_options");
    GtkWidget *toolbar_home = lookup_widget(widget,
					    "toolbar_home");
    GtkWidget *toolbar_back = lookup_widget(widget,
					    "toolbar_back");
    GtkWidget *toolbar_forward = lookup_widget(widget,
					       "toolbar_forward");
    GtkWidget *back_to_main = lookup_widget(widget,
					    "main_buttons_back_to_main");
    GtkWidget *enter_dogs_in_races = lookup_widget(widget,
						   "main_buttons_enter_dogs");
    GtkWidget *go_to_races = lookup_widget(widget,
					   "main_buttons_race");
    GtkWidget *show_race_results = lookup_widget(widget,
						 "main_buttons_results");
    GtkWidget *trading = lookup_widget(widget,
				       "main_buttons_trading");
    GtkWidget *breeding = lookup_widget(widget,
					"main_buttons_breeding");
    GtkWidget *show_puppies = lookup_widget(widget,
					    "show_puppies");
    GtkWidget *enter_dog_in_race = lookup_widget(widget,
						 "dog_buttons_enter_dog");
    GtkWidget *remove_dog_from_race = lookup_widget(widget,
						    "dog_buttons_remove_dog");
    GtkWidget *trade_dogs = lookup_widget(widget,
					  "dog_buttons_trade_dogs");
    GtkWidget *mate_dogs = lookup_widget(widget,
					 "dog_buttons_mate_dogs");
    GtkWidget *promote_pup = lookup_widget(widget,
					   "dog_buttons_promote_pup");
    GtkWidget *give_away_dog = lookup_widget(widget,
					     "dog_buttons_give_away");

    GtkWidget *statistics = lookup_widget(widget,
					  "main_buttons_statistics");
    GtkWidget *save_status = lookup_widget(widget, "save_status");
    
    gint status = get_int_from_label(GTK_LABEL(save_status));

    GtkWidget *current_kennel_id = lookup_widget(widget,
						 "current_kennel_id");
    gint kennel_id = get_int_from_label(GTK_LABEL(current_kennel_id));
    
    GtkWidget *dog_info_label = lookup_widget(widget,
					      "dog_info_label");
    gchar *info_label;
    
    GtkWidget *dog_info_label_2 = lookup_widget(widget,
						"dog_info_label_2");
    gchar *info_label_2;
    GtkWidget *dog_info_label_3 = lookup_widget(widget,
						"dog_info_label_3");
    gchar *info_label_3;
    GtkWidget *dog_info_label_4 = lookup_widget(widget,
						"dog_info_label_4");
    gchar *info_label_4;
    GtkWidget *dog_info_label_5 = lookup_widget(widget,
						"dog_info_label_5");
    gchar *info_label_5;
    GtkWidget *dog_info_label_6 = lookup_widget(widget,
						"dog_info_label_6");
    gchar *info_label_6;

    gint i;
    gint dog_indices[2];
    gint temporary_gint;
    
    /* set all buttons to non-sensitive;
       exception: the 'forward' and 'back' buttons
       which are set to sensitive */
    set_buttons_non_sensitive(widget);

    /* get dog indices */
    get_dog_indices(widget, dog_indices);

    /* first have a look at the dog_info labels: */
    gtk_label_get(GTK_LABEL(dog_info_label), &info_label);
    gtk_label_get(GTK_LABEL(dog_info_label_2), &info_label_2);
    gtk_label_get(GTK_LABEL(dog_info_label_3), &info_label_3);
    gtk_label_get(GTK_LABEL(dog_info_label_4), &info_label_4);
    gtk_label_get(GTK_LABEL(dog_info_label_5), &info_label_5);
    gtk_label_get(GTK_LABEL(dog_info_label_6), &info_label_6);    

    /* if we aren't looking a race, we can switch on all the
       menu buttons except the main-menu button */
    if(strcmp(info_label, " Race results  Level: ") != 0)
    {
	gtk_widget_set_sensitive(toolbar_quit, TRUE);
	gtk_widget_set_sensitive(toolbar_options, TRUE);
	gtk_widget_set_sensitive(toolbar_load, TRUE);
	if(status != 1)
	    gtk_widget_set_sensitive(toolbar_save, TRUE);
	gtk_widget_set_sensitive(enter_dogs_in_races, TRUE);
	gtk_widget_set_sensitive(go_to_races, TRUE);
	
	/* in the very first week in the game there are no race
	   results */
	if(season != 1 || month != 1 || week != 1)
	    gtk_widget_set_sensitive(show_race_results, TRUE);
	
	gtk_widget_set_sensitive(trading, TRUE);
	gtk_widget_set_sensitive(breeding, TRUE);
	gtk_widget_set_sensitive(show_puppies, TRUE);
	gtk_widget_set_sensitive(statistics, TRUE);
	gtk_option_menu_set_history(GTK_OPTION_MENU(statistics), 0);
    }

    /* now: normal mode, just looking at kennels */
    if(strcmp(info_label, " Detailed dog information") == 0)
    {
	/* everything is off except 'give_away' if it's the puppy or
	   home kennel and 'home' if it's not the home kennel */
	if(kennel_id != 1)
	    gtk_widget_set_sensitive(toolbar_home, TRUE);

	if( (kennel_id == 1 || kennel_id == 21) &&
	    dog_indices[0] != -1)
	    gtk_widget_set_sensitive(give_away_dog, TRUE);

	if(kennel_id != 21)
	    gtk_widget_set_sensitive(show_puppies, TRUE);

	if(kennel_id == 21)
	{
	    gtk_widget_set_sensitive(show_puppies, FALSE);
	    
	    /* maybe a pup can be promoted */
	    if(dog_indices[0] != -1 &&
	       dogs[dog_indices[0]].age > 9 &&
	       dogs_in_kennel(0) < 20)
		
		gtk_widget_set_sensitive(promote_pup, TRUE);
	}
    }    
    
    /* entering dogs in races */
    else if(strcmp(info_label, " Entering dogs  Level: ") == 0)
    {
	/* back to main: on */
	gtk_widget_set_sensitive(back_to_main, TRUE);
	/* entering dogs: off */
	gtk_widget_set_sensitive(enter_dogs_in_races, FALSE);

	/* find out which race */
	temporary_gint = get_int_from_label(GTK_LABEL(dog_info_label_6)) - 1;

	/* can the dog be entered?
	   if we have trophy races, we have to
	   check whether the human player is qualified
	   and besides we ignore level binding */
	gtk_widget_set_sensitive(
	    enter_dog_in_race,
	    set_sensitive_enter_dogs(dog_indices[0], temporary_gint));

	/* if a dog can be removed */
	for(i=0;i<10;i++)
	    if(races[1][temporary_gint].dogs[i] != NULL &&
	       races[1][temporary_gint].dogs[i]->id < 20)
	    {
		gtk_widget_set_sensitive(remove_dog_from_race, TRUE);
		break;
	    }
    }

    /* looking at races at the end of the semi-month */
    else if(strcmp(info_label, " Race results  Level: ") == 0)
    {
	/* everything is off except back to main */
	gtk_widget_set_sensitive(back_to_main, TRUE);
	
	/* as a special tension-building feature when
	   it's trophy racing, we only allow 'forward' */
	if(strcmp(info_label_2, "any") == 0)
	    gtk_widget_set_sensitive(toolbar_back, FALSE);
    }

    /* looking at race results during a semi-months activities */
    else if(strcmp(info_label, " Last races  Level: ") == 0)
    {
	gtk_widget_set_sensitive(show_race_results, FALSE);
    	gtk_widget_set_sensitive(back_to_main, TRUE);
    }

    /* trading */
    else if(strcmp(info_label, " Trading with  Kennel: ") == 0)
    {
	gtk_widget_set_sensitive(trading, FALSE);
	gtk_widget_set_sensitive(back_to_main, TRUE);
	gtk_widget_set_sensitive(trade_dogs, TRUE);
    }

    /* breeding */
    else if(strcmp(info_label, " Mating dogs  Kennel: ") == 0)
    {
	gtk_widget_set_sensitive(breeding, FALSE);
	gtk_widget_set_sensitive(back_to_main, TRUE);
	gtk_widget_set_sensitive(mate_dogs,
				 set_sensitive_mate_dogs(widget));
    }
    /* statistics */
    else if(strncmp(info_label, " Statistics", 6) == 0)
    {
	gtk_widget_set_sensitive(back_to_main, TRUE);

	if(strncmp(info_label_2, "Champ", 4) == 0 ||
	   strncmp(info_label_2, "All", 2) == 0 ||
	   (strncmp(info_label_2, "Hist", 3) == 0 && season < 3) )
	{
	    gtk_widget_set_sensitive(toolbar_back, FALSE);
	    gtk_widget_set_sensitive(toolbar_forward, FALSE);	    
	}
    }

    return 0;
}

/* return TRUE if the dog may participate in the race (fitting level,
   status etc.), FALSE else */
gboolean set_sensitive_enter_dogs(gint dog_index, gint race_number)
{
    if(dog_index != -1)	
	/* the dog's gotta be healthy */
	if( entered_in_races(dogs[dog_index]) == 0 &&
	    dogs[dog_index].status < 20 &&
	    /* this is because very young bitches
	       aren't in heat */
	    (dogs[dog_index].status != 0 || 
	     (dogs[dog_index].status == 0 &&
	      dogs[dog_index].age < 12) ) &&
	    /* either it's a normal race and we take care
	       about dog level / race level */
	    ( (dogs[dog_index].level % 10 == 
	       races[1][race_number].level &&
	       (month != 12 || week != 3)  ) ||
	      /* or it's a trophy race and the home kennel
		 has to be qualified */
	      (is_in_list(qualified_kennels, 0, 5, 0) == 1 &&
	       month == 12 && week == 3) ) )
	    return TRUE;

    return FALSE;
}

/* justify the (first) eight columns of 'clist' according to the
   integer values.
   col = 0: justify left
   col = 1: centered
   col = 2: right */
gint justify_clist(GtkCList *clist, gint col_1, gint col_2, gint col_3,gint col_4,
		  gint col_5,gint col_6,gint col_7,gint col_8)
{
    if(col_1 == 0)
	gtk_clist_set_column_justification(clist, 0, GTK_JUSTIFY_LEFT);
    else if(col_1 == 1)
	gtk_clist_set_column_justification(clist, 0, GTK_JUSTIFY_CENTER);
    else if(col_1 == 2)
	gtk_clist_set_column_justification(clist, 0, GTK_JUSTIFY_RIGHT);

    if(col_2 == 0)
	gtk_clist_set_column_justification(clist, 1, GTK_JUSTIFY_LEFT);
    else if(col_2 == 1)
	gtk_clist_set_column_justification(clist, 1, GTK_JUSTIFY_CENTER);
    else if(col_2 == 2)
	gtk_clist_set_column_justification(clist, 1, GTK_JUSTIFY_RIGHT);

    if(col_3 == 0)
	gtk_clist_set_column_justification(clist, 2, GTK_JUSTIFY_LEFT);
    else if(col_3 == 1)
	gtk_clist_set_column_justification(clist, 2, GTK_JUSTIFY_CENTER);
    else if(col_3 == 2)
	gtk_clist_set_column_justification(clist, 2, GTK_JUSTIFY_RIGHT);

    if(col_4 == 0)
	gtk_clist_set_column_justification(clist, 3, GTK_JUSTIFY_LEFT);
    else if(col_4 == 1)
	gtk_clist_set_column_justification(clist, 3, GTK_JUSTIFY_CENTER);
    else if(col_4 == 2)
	gtk_clist_set_column_justification(clist, 3, GTK_JUSTIFY_RIGHT);

    if(col_5 == 0)
	gtk_clist_set_column_justification(clist, 4, GTK_JUSTIFY_LEFT);
    else if(col_5 == 1)
	gtk_clist_set_column_justification(clist, 4, GTK_JUSTIFY_CENTER);
    else if(col_5 == 2)
	gtk_clist_set_column_justification(clist, 4, GTK_JUSTIFY_RIGHT);

    if(col_6 == 0)
	gtk_clist_set_column_justification(clist, 5, GTK_JUSTIFY_LEFT);
    else if(col_6 == 1)
	gtk_clist_set_column_justification(clist, 5, GTK_JUSTIFY_CENTER);
    else if(col_6 == 2)
	gtk_clist_set_column_justification(clist, 5, GTK_JUSTIFY_RIGHT);

    if(col_7 == 0)
	gtk_clist_set_column_justification(clist, 6, GTK_JUSTIFY_LEFT);
    else if(col_7 == 1)
	gtk_clist_set_column_justification(clist, 6, GTK_JUSTIFY_CENTER);
    else if(col_7 == 2)
	gtk_clist_set_column_justification(clist, 6, GTK_JUSTIFY_RIGHT);

    if(col_8 == 0)
	gtk_clist_set_column_justification(clist, 7, GTK_JUSTIFY_LEFT);
    else if(col_8 == 1)
	gtk_clist_set_column_justification(clist, 7, GTK_JUSTIFY_CENTER);
    else if(col_8 == 2)
	gtk_clist_set_column_justification(clist, 7, GTK_JUSTIFY_RIGHT);

    return 0;
}

/* if the season is over except the trophy races and
   the human player's first in one of the kennel categories,
   note it in the two 'selected_row' labels */
gint awards_news(GtkWidget *widget)
{
    GtkWidget *misc_events = 
	lookup_widget(widget, "misc_events");
    
    gint i;
    gint info = 0;
    gint temporary_order[20];

    /* fastest times */
    for(i=0;i<3;i++)
    {
	sort_kennel_statistics((i + 2) * 10, temporary_order, 0);
	/* note it if the human player's kennel is first */
	if(temporary_order[0] == 0)
	    info += pow(2, (double)i);
    }
    
    /* champions' race categories */
    for(i=0;i<3;i++)
    {
	sort_kennel_statistics((i + 9) * 10, temporary_order, 0);
	if(temporary_order[0] == 0)
	    info += pow(2, (double)i) * 10;
    }

    label_set_text_from_int(GTK_LABEL(misc_events), info, 0);

    return 0;
}

/* show various popups with news for the user at the
   beginning of a new race-weekend */
gint news(GtkWidget *widget)
{
    /* puppy news */
    GtkWidget *puppy_info = lookup_widget(widget, "puppy_info");
    /* dog news */
    GtkWidget *dogs_info = lookup_widget(widget, "dogs_info");
    /* awards news at the end of the season */
    GtkWidget *misc_events = lookup_widget(widget, 
					   "misc_events");   
    gchar temporary_gchar[80];
    gint info = get_int_from_label(GTK_LABEL(puppy_info));
    
    /* puppy news */
    if(decompose_compound_integer(1, info) == 1 && options_show_news[0] == 1)
	print_news(widget,
		   "Congratulations! New puppies were born! ", 0);
    if(decompose_compound_integer(2, info) == 1 && options_show_news[1] == 1)
	print_news(widget,"I'm sorry to tell you that there was not enough room for all of your new puppies. Some had to be given away. ", 1);   
    if(decompose_compound_integer(4, info) == 1 && options_show_news[2] == 1)
	print_news(widget, "Some of your puppies are old enough to be dogs now. You should promote them or they will be promoted or given away (if there's no room in your kennel) automatically next time. ", 2);
    if(decompose_compound_integer(8, info) == 1 && options_show_news[3] == 1)
	print_news(widget, "Some of your puppies had to be given away as they were too old and there was no room for them in your kennel. ", 3);
    /* reset the puppy info label */
    gtk_label_set_text(GTK_LABEL(puppy_info), "0");    

    /* dogs news */
    info = get_int_from_label(GTK_LABEL(dogs_info));
    if(decompose_compound_integer(1, info % 10) == 1 &&
       options_show_news[4] == 1)
	print_news(widget, "Some of your dogs were too old for racing. Good homes and loving families have been found for them.", 4);
    if(decompose_compound_integer(2, info % 10) == 1 && 
       options_show_news[5] == 1)
	print_news(widget, "There was fighting in your kennel. Perhaps because of bitches in heat.", 5);
    if(info > 9)
    {
	sprintf(temporary_gchar, "There is a bitch in heat in kennel %d that might interest you.", (info - info % 10) / 10 + 1);
	print_news(widget, temporary_gchar, 7);
    }
    gtk_label_set_text(GTK_LABEL(dogs_info), "0");    

    /* awards news */
    if(month == 12 && week == 3)
    {
	info = get_int_from_label(GTK_LABEL(misc_events));
	info = info % 10;
	if(decompose_compound_integer(1, info) == 1 &&
	   options_show_news[6] == 1)
	    print_news(widget, "Congratulations! You have won the Strelcov Trophy for the fastest 1-lap race this season!", 6);
	if(decompose_compound_integer(2, info) == 1 &&
	   options_show_news[6] == 1)
	    print_news(widget, "Congratulations! You have won the Psmith Memorial Award for the fastest 2-lap race this season!", 6);
	if(decompose_compound_integer(4, info) == 1 && 
	   options_show_news[6] == 1)
	    print_news(widget, "Congratulations! You have won the Silenus Award for the fastest 3-lap race this season!", 6);

	info = get_int_from_label(GTK_LABEL(misc_events));
	info = (info - info % 10) / 10;
	if(decompose_compound_integer(1, info) == 1 &&
	   options_show_news[6] == 1)
	    print_news(widget, "Congratulations! You have won the von Neumann Memorial Trophy for the best Weighted Average Time this season!", 6);
	if(decompose_compound_integer(2, info) == 1 && 
	   options_show_news[6] == 1)
	    print_news(widget, "Congratulations! You have won the Wacklewood Trophy for most points per race this season!", 6);
	if(decompose_compound_integer(4, info) == 1 && 
	   options_show_news[6] == 1)
	    print_news(widget, "Congratulations! You have won the Illyan Award for the best winning rate this season!", 6);
	gtk_label_set_text(GTK_LABEL(misc_events), "0");
    }

    /* champions' trophy */
    info = get_int_from_label(GTK_LABEL(misc_events));
    if(decompose_compound_integer(1, info) == 1)
	popup_trophy_winner(widget);
    gtk_label_set_text(GTK_LABEL(misc_events), "0");
    
    return 0;
}

/* show a popup with the trophy winner */
gint popup_trophy_winner(GtkWidget *widget)
{
    gchar temporary_gchar[100];    
    history_element *element = first_history;

    if(element == NULL)
    {
	g_print("popup_tropy_winner: history element is NULL\n");
	return 1;
    }
    
    /* find most recent history */
    while(element->next != NULL)
	element = element->next;

    /* user has won */
    if(strcmp(element->kennel_names[0], kennels[0].name) == 0)
	print_news(widget, "Congratulations! You have won the Champions' Trophy!", 60);
    else
    {
	sprintf(temporary_gchar, "%s has won the Champions' Trophy.",
		element->kennel_names[0]);
	print_news(widget, temporary_gchar, 60);
    }

    return 0;
}

/* show a small window with a progress bar advancing every 1/20 of a second
   'duration' times. 'text' is written above the progress bar */
gint show_progress(gint duration, gchar *text)
{
    GtkWidget *progress_window = create_progress_window();
    GtkWidget *progress_bar = lookup_widget(progress_window, 
					    "progress_window_progress_bar");
    GtkWidget *progress_text = lookup_widget(progress_window,
					     "progress_window_text");

    gfloat percentage = 0;
    gint i;
    
    gtk_label_set(GTK_LABEL(progress_text), text);

    gtk_widget_show(progress_window);

    for(i=0;i<duration * 2;i++)
    {
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(progress_bar),
				      percentage);
	
	while(gtk_events_pending())
	    gtk_main_iteration();

	percentage += 1 / (gfloat)(duration * 2 - 1);
	
	usleep(50000);
    }

    gtk_widget_destroy(progress_window);

    return 0;
}

/* show an ok_or_cancel window with text 'text' and type 'type' */
gint show_ok_or_cancel_window(GtkWidget *widget, gchar *text, gint type)
{
    GtkWidget *ok_or_cancel_window = create_ok_or_cancel_window();
    GtkWidget *main_window = lookup_widget(widget, "main_window");
    GtkWidget *ok_or_cancel_question =
	lookup_widget(GTK_WIDGET(ok_or_cancel_window), "ok_or_cancel_question");
    GtkWidget *ok_or_cancel_type =
	lookup_widget(GTK_WIDGET(ok_or_cancel_window), "ok_or_cancel_type");
    GtkWidget *ok_or_cancel_check_button = 
	lookup_widget(GTK_WIDGET(ok_or_cancel_window),
		      "ok_or_cancel_check_button");
    GtkWidget *ok_or_cancel_alternative = 
	lookup_widget(GTK_WIDGET(ok_or_cancel_window),
		      "ok_or_cancel_alternative");
    GtkWidget *ok_or_cancel_image = 
	lookup_widget(GTK_WIDGET(ok_or_cancel_window),
		      "ok_or_cancel_image");
    gchar temporary_gchar[5];

    gtk_label_set_text(GTK_LABEL(ok_or_cancel_question), text);
    
    gtk_widget_show( ok_or_cancel_window );        

    /* show alternative button if we have 'quit?', or 'load game?'
       at the beginning */
    if(type == 50 || type == 100)
    {
	gtk_widget_hide(ok_or_cancel_check_button);

	if(type == 50)
	    gtk_label_set_text(
		GTK_LABEL(GTK_BIN(ok_or_cancel_alternative)->child ),
		"Save and Quit" );
	else if(type == 100)
	    gtk_label_set_text(
		GTK_LABEL(GTK_BIN(ok_or_cancel_alternative)->child ),
		"No, start new game" );
	
	gtk_widget_show(ok_or_cancel_alternative);
    }

    /* set the correct image */
    if(type == 0)
	gtk_image_set_from_file(GTK_IMAGE(ok_or_cancel_image),
				my_find_pixmap_file(
				    "dog_buttons_give_away.xpm"));
    else if(type == 1)
	gtk_image_set_from_file(GTK_IMAGE(ok_or_cancel_image),
				my_find_pixmap_file(
				    "main_buttons_race.xpm"));
    else if(type == 2 || type == 3)
	gtk_image_set_from_file(GTK_IMAGE(ok_or_cancel_image),
				my_find_pixmap_file(
				    "dog_buttons_mate_dogs.xpm"));
    else if(type == 100)
	gtk_image_set_from_file(GTK_IMAGE(ok_or_cancel_image),
				my_find_pixmap_file(
				    "toolbar_load.xpm"));
    else
	gtk_image_set_from_file(GTK_IMAGE(ok_or_cancel_image),
				my_find_pixmap_file(
				    "events_warning.png"));
    
    /* set question type for the ok_or_cancel window */
    label_set_text_from_int(GTK_LABEL(ok_or_cancel_type), type, 0);
    
    change_popups_active(main_window, 1);

    return 0;
}

/* determine which season history to show and go ahead */
gint show_history(GtkWidget *widget, gint seasonnr)
{
    gint i;
    gint dog_info_width[8] = {165, 250, 0, 0, 0, 0, 0, 0};
    gchar dog_info_title[8][16];
    gchar *info_label;
    history_element *to_show = first_history;
    GtkWidget *dog_info_label_2 = 
	lookup_widget(widget, "dog_info_label_2");
    GtkWidget *dog_info_label_3 = 
	lookup_widget(widget, "dog_info_label_3");

    /* find out whether the function's been called
       while turning the history pages or clicking
       on 'statistics' */
    gtk_label_get(GTK_LABEL(dog_info_label_2), &info_label);

    if(strncmp(info_label, "Hist", 3) == 0)
	seasonnr +=  get_int_from_label(GTK_LABEL(dog_info_label_3));

    /* if seasonnr is 0, we show the first season */
    /* and when the user turns the page to the right 
       and we already display the most recent season,
       we start at season 1 */
    if(seasonnr == season)
    {
	to_show = first_history;
	seasonnr = 1;
    }
    else if(seasonnr == 0)
	seasonnr = season - 1;

    to_show = first_history;
    for(i=0;i < seasonnr - 1;i++)
	to_show = to_show->next;
    
    format_dog_info(widget, dog_info_width, dog_info_title,
		    seasonnr, 0, 0, 80);

    show_season_history(widget, to_show, seasonnr);
    
    return 0;
}

/* show a page of history (from a history_element) in dog_info */
gint show_season_history(GtkWidget *widget, history_element *element,
			gint seasonnr)
{
    GtkWidget *dog_info = lookup_widget(widget, "dog_info");
    gint i;
    gchar *row[8];
    gchar temporary_gchar[50] = "\0";

    gtk_clist_freeze(GTK_CLIST(dog_info));
    gtk_clist_clear(GTK_CLIST(dog_info));
    
    if(element == NULL || season == 1)
    {
	namescan( &row[0], "This is your first season.");
	for(i=1;i<8;i++)
	    namescan( &row[i], "");
	gtk_clist_append(GTK_CLIST(dog_info), row);
	for(i=0;i<8;i++)
	    g_free(row[i]);

	gtk_clist_thaw(GTK_CLIST(dog_info));
	
	return 1;
    }

    for(i=2;i<8;i++)
	namescan( &row[i], "");
    
   /* champion */
    namescan( &row[0], "Champions' Trophy Winner:");
    sprintf(temporary_gchar, "%s (%.0f points)",
	    element->kennel_names[0], element->kennel_values[0]);
    namescan( &row[1], temporary_gchar);
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);
    
    /* the three trophy winners */
    namescan( &row[0], "von Neumann");
    namescan( &row[1], "");
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);
    
    namescan( &row[0], "Memorial Trophy Winner:");
    sprintf(temporary_gchar, "%s (%.2f WA Time)",
	    element->kennel_names[1],
	    element->kennel_values[1]);
    namescan( &row[1], temporary_gchar);
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);

    namescan( &row[0], "Wacklewood Trophy Winner:");
    sprintf(temporary_gchar, "%s (%.2f points / race)",
	    element->kennel_names[2],
	    element->kennel_values[2]);
    namescan( &row[1], temporary_gchar);
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);

    namescan( &row[0], "Illyan Award Winner:");
    sprintf(temporary_gchar, "%s (%.2f wins / race)",
	    element->kennel_names[3],
	    element->kennel_values[3]);
    namescan( &row[1], temporary_gchar);
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);

    /* highlight rows if the user's in them */
    if(strcmp(element->kennel_names[0], kennels[0].name) == 0)
	change_color_row(GTK_CLIST(dog_info),
			 clist_last_row(GTK_CLIST(dog_info)), 20);
    for(i=2;i<5;i++)
	if(strcmp(element->kennel_names[i - 1], kennels[0].name) == 0)
	    change_color_row(GTK_CLIST(dog_info), i, 20);

    namescan( &row[0], ""); namescan( &row[1], "");
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);

    
    namescan( &row[0], "Best dogs:"); namescan( &row[1], "");
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);
    
    for(i=0;i<3;i++)
    {
	if(i == 0)
	    namescan( &row[0], "Fastest 1-lap race:");
	else if(i == 1)
	    namescan( &row[0], "Fastest 2-lap race:");
	else
	    namescan( &row[0], "Fastest 3-lap race:");
	
	if(element->dog_values[i] < 100)
	    sprintf(temporary_gchar, "%.2f %s (%s)", element->dog_values[i],
		    element->dog_names[i], element->dog_kennel_names[i]);
	else
	    sprintf(temporary_gchar, "--:-- %s (%s)",
		    element->dog_names[i], element->dog_kennel_names[i]);
	
	namescan( &row[1], temporary_gchar);
	gtk_clist_append(GTK_CLIST(dog_info), row);
	g_free(row[0]);
	g_free(row[1]);
    }


    for(i=3;i<5;i++)
    {
	if(i == 3)
	    namescan( &row[0], "Most points:");
	else
	    namescan( &row[0], "Most wins:");
	
	sprintf(temporary_gchar, "%.0f %s (%s)", element->dog_values[i],
		element->dog_names[i], element->dog_kennel_names[i]);
	namescan( &row[1], temporary_gchar);
	gtk_clist_append(GTK_CLIST(dog_info), row);
	g_free(row[0]);
	g_free(row[1]);
    }

    for(i=5;i<7;i++)
    {
	if(i == 5)
	    namescan( &row[0], "Most points / race:");
	else
	    namescan( &row[0], "Most wins / race:");
	
	sprintf(temporary_gchar, "%.2f %s (%s)", element->dog_values[i],
		element->dog_names[i], element->dog_kennel_names[i]);
	namescan( &row[1], temporary_gchar);
	gtk_clist_append(GTK_CLIST(dog_info), row);
	g_free(row[0]);
	g_free(row[1]);
    }

    /* highlight */
    for(i=7;i<14;i++)
	if(strcmp(element->dog_kennel_names[i - 7], kennels[0].name) == 0)
	    change_color_row(GTK_CLIST(dog_info), i, 20);

    namescan( &row[0], ""); namescan( &row[1], "");
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);

    namescan( &row[0], "Your kennel:"); namescan( &row[1], "");
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);

    namescan( &row[0], "Champions' Trophy ranking:");
    if(element->user_kennel_results[0] == -1)
	namescan( &row[1], "not qualified");
    else
    {
	sprintf(temporary_gchar, "%d (%.0f points)",
		element->user_kennel_results[0],
		element->user_kennel_values[0]);
	namescan( &row[1], temporary_gchar);
    }
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);

    namescan( &row[0], "von Neumann");
    namescan( &row[1], "");
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);
    
    namescan( &row[0], "Memorial Trophy ranking:");
    if(element->user_kennel_values[1] < 70)
	sprintf(temporary_gchar, "%d (%.2f Weighted Average Time)",
		element->user_kennel_results[1],
		element->user_kennel_values[1]);
    else
	sprintf(temporary_gchar, "%d (no Weighted Average Time)",
		element->user_kennel_results[1]);

    namescan( &row[1], temporary_gchar);
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);

    namescan( &row[0], "Wacklewood Trophy ranking:");
    sprintf(temporary_gchar, "%d (%.2f points / race)",
	    element->user_kennel_results[2],
	    element->user_kennel_values[2]);
    namescan( &row[1], temporary_gchar);
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);

    namescan( &row[0], "Illyan Award ranking:");
    sprintf(temporary_gchar, "%d (%.2f wins / race)",
	    element->user_kennel_results[3],
	    element->user_kennel_values[3]);
    namescan( &row[1], temporary_gchar);
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);

    namescan( &row[0], ""); namescan( &row[1], "");
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);

    namescan( &row[0], "Your fastest dogs:"); namescan( &row[1], "");
    gtk_clist_append(GTK_CLIST(dog_info), row);
    g_free(row[0]);
    g_free(row[1]);
    
    for(i=0;i<3;i++)
    {
	if(i == 0)
	    namescan( &row[0], "Fastest 1-lap race:");
	else if(i == 1)
	    namescan( &row[0], "Fastest 2-lap race:");
	else
	    namescan( &row[0], "Fastest 3-lap race:");
	
	if(element->user_dog_values[i] < 100)
	    sprintf(temporary_gchar, "%.2f %s", element->user_dog_values[i],
		    element->user_dog_names[i]);
	else
	    sprintf(temporary_gchar, "--:--");
	
	namescan( &row[1], temporary_gchar);
	gtk_clist_append(GTK_CLIST(dog_info), row);
	g_free(row[0]);
	g_free(row[1]);
    }

    gtk_clist_thaw(GTK_CLIST(dog_info));
    
    return 0;
}

/* type = 0: set the buttons in the options window corresponding to
             the values of the global option variables when the options
	     window is popped up
   type = 10: set the values corresponding to the buttons when the user
              clicks 'OK' in the options window */
gint set_buttons_options_window(GtkWidget *widget, gint type)
{
    /* the array indices here correspond to the
       array indices of the global options variables */
    GtkWidget *zero_one_options[21];
    GtkWidget *option_misc_notif[2];
    GtkWidget *option_misc_notif_spinbutton[2];
    GtkWidget *options_notify_radio_or;
    GtkWidget *options_notify_radio_and;
    GtkWidget *race_options_radio[3];
    GtkWidget *race_options_spinbutton;

    gint *value;
    gint i, limit;

    zero_one_options[0] = lookup_widget(widget, "options_confirm0");
    zero_one_options[1] = lookup_widget(widget, "options_confirm1");
    zero_one_options[2] = lookup_widget(widget, "options_confirm2");
    zero_one_options[3] = lookup_widget(widget, "options_confirm3");
    zero_one_options[4] = lookup_widget(widget, "options_confirm4");

    zero_one_options[5] = lookup_widget(widget, "options_news0");
    zero_one_options[6] = lookup_widget(widget, "options_news1");
    zero_one_options[7] = lookup_widget(widget, "options_news2");
    zero_one_options[8] = lookup_widget(widget, "options_news3");
    zero_one_options[9] = lookup_widget(widget, "options_news4");
    zero_one_options[10] = lookup_widget(widget, "options_news5");
    zero_one_options[11] = lookup_widget(widget, "options_news6");
    
    zero_one_options[12] = lookup_widget(widget, "options_misc1");
    zero_one_options[13] = lookup_widget(widget, "options_misc2");
    zero_one_options[14] = lookup_widget(widget, "options_misc3");    

    zero_one_options[15] = lookup_widget(widget, "live_race_check0");
    zero_one_options[16] = lookup_widget(widget, "live_race_check1");
    zero_one_options[17] = lookup_widget(widget, "live_race_check2");
    zero_one_options[18] = lookup_widget(widget, "live_race_check3");
    zero_one_options[19] = lookup_widget(widget, "live_race_check4");
    zero_one_options[20] = lookup_widget(widget, "live_race_check5");

    race_options_radio[0] = lookup_widget(widget, "options_misc0_0");
    race_options_radio[1] = lookup_widget(widget, "options_misc0_1");
    race_options_radio[2] = lookup_widget(widget, "options_misc0_2");
    race_options_spinbutton = lookup_widget(widget, "options_misc0_spinbutton");

    option_misc_notif[0] = lookup_widget(widget, "options_misc18");
    option_misc_notif[1] = lookup_widget(widget, "options_misc19");

    option_misc_notif_spinbutton[0] = 
	lookup_widget(widget, "options_misc18_spinbutton");
    option_misc_notif_spinbutton[1] = 
	lookup_widget(widget, "options_misc19_spinbutton");
    options_notify_radio_or = 
	lookup_widget(widget, "options_notify_radio_or");;
    options_notify_radio_and = 
	lookup_widget(widget, "options_notify_radio_and");

    /* set zero_one buttons correctly */
    for(i=0;i<21;i++)
    {
	if(i < 5)
	    value = &options_confirm[i];
	else if(i < 12)
	    value = &options_show_news[i - 5];
	else if(i < 15)
	    value = &options_misc[i - 11];
	else if(i < 21)
	    value = &options_misc[i - 10];

	if(type == 0)
	{
	    if(*value == 0)
	    {
		gtk_toggle_button_set_active(
		    GTK_TOGGLE_BUTTON(zero_one_options[i]), FALSE);
	    }
	    else
		gtk_toggle_button_set_active(
		    GTK_TOGGLE_BUTTON(zero_one_options[i]), TRUE);
	}
	else
	{
	    if(gtk_toggle_button_get_active(
		   GTK_TOGGLE_BUTTON(zero_one_options[i])))
		*value = 1;
	    else
		*value = 0;
	}
    }

    /* the 'live' race stuff */
    if(type == 0)
    {	
	if(options_misc[0] > 199)
	{
	    gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(race_options_radio[2]), TRUE);
	    gtk_widget_set_sensitive(
		race_options_spinbutton->parent, FALSE);
	    gtk_widget_set_sensitive(
		zero_one_options[15]->parent, FALSE);
	}
	else if(options_misc[0] > 99)
	{
	    gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(race_options_radio[1]), TRUE);
	    gtk_widget_set_sensitive(
		race_options_spinbutton->parent, FALSE);
	    gtk_widget_set_sensitive(
		zero_one_options[15]->parent, FALSE);
	}
	else
	{
	    gtk_toggle_button_set_active(
		GTK_TOGGLE_BUTTON(race_options_radio[0]), TRUE);
	    gtk_widget_set_sensitive(
		race_options_spinbutton->parent, TRUE);
	    gtk_widget_set_sensitive(
		zero_one_options[15]->parent, TRUE);
	    gtk_spin_button_set_value(
		GTK_SPIN_BUTTON(race_options_spinbutton),
		(double)(options_misc[0] % 100));
	}	
    }
    else
    {
	/* if the 'show live race on level' checkbuttons
	   are all off, we switch on 'show race progress' */
	limit = 0;
	for(i=5;i<11;i++)
	    limit += options_misc[i];

	if( gtk_toggle_button_get_active(
		GTK_TOGGLE_BUTTON(race_options_radio[2])) )
	    options_misc[0] = options_misc[0] % 100 + 200;
	else if( gtk_toggle_button_get_active(
		     GTK_TOGGLE_BUTTON(race_options_radio[1])) ||
		 (gtk_toggle_button_get_active(
		     GTK_TOGGLE_BUTTON(race_options_radio[0])) &&
		  limit == 0) )
	{
	    options_misc[0] = options_misc[0] % 100 + 100;
	    options_misc[10] = 1;
	}
	else
	{
	    limit = gtk_spin_button_get_value_as_int(
		GTK_SPIN_BUTTON(race_options_spinbutton));

	    if(limit > -1 && limit < 100)
		options_misc[0] = limit;
	}
    }
    
    /* the two notify limit buttons */
    for(i=0;i<2;i++)
	if(type == 0)
	{
	    if(options_misc[i + 18] >= 100)
	    {
		gtk_toggle_button_set_active(
		    GTK_TOGGLE_BUTTON(option_misc_notif[i]), FALSE);
		gtk_widget_set_sensitive(option_misc_notif_spinbutton[i],
					 FALSE);
	    }
	    else if(options_misc[i + 18] < 100)
	    {
		gtk_toggle_button_set_active(
		    GTK_TOGGLE_BUTTON(option_misc_notif[i]), TRUE);
		gtk_widget_set_sensitive(option_misc_notif_spinbutton[i],
					 TRUE);
		gtk_spin_button_set_value(
		    GTK_SPIN_BUTTON(option_misc_notif_spinbutton[i]),
		    (double)(options_misc[i + 18] % 100));
	    }
	}
	else
	{
	    limit = gtk_spin_button_get_value_as_int(
		GTK_SPIN_BUTTON(option_misc_notif_spinbutton[i]));

	    if(gtk_toggle_button_get_active(
		   GTK_TOGGLE_BUTTON(option_misc_notif[i])))
	    {
		if(limit > 0)
		    options_misc[i + 18] = limit;
		else
		    gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(option_misc_notif[i]), FALSE);
	    }
	    else
		options_misc[i + 18] = options_misc[i + 18] % 100 + 100;
	}

    /* the conjunction between the two limits */
    if(type == 0)
    {
	if(options_misc[18] >= 100 || options_misc[19] >= 100)
	    gtk_widget_set_sensitive(options_notify_radio_or->parent,
				     FALSE);
	else
	{
	    gtk_widget_set_sensitive(options_notify_radio_or->parent,
				     TRUE);
	    if(options_misc[17] == 1)
		gtk_toggle_button_set_active(
		    GTK_TOGGLE_BUTTON(options_notify_radio_or), TRUE);
	    else
		gtk_toggle_button_set_active(
		    GTK_TOGGLE_BUTTON(options_notify_radio_and), TRUE);
	}
    }
    else
    {
	if(gtk_toggle_button_get_active(
	       GTK_TOGGLE_BUTTON(options_notify_radio_and)))
	    options_misc[17] = 0;
	else
	    options_misc[17] = 1;
    }

    return 0;
}

/* return the last row of a clist that
   contains something */
gint clist_last_row(GtkCList *clist)
{
    gint i, last_row = -1;
    gchar *temporary_gchar = NULL;

    for(i=0;i<30;i++)
    {
	gtk_clist_get_text(clist, i, 0, &temporary_gchar);
	if(temporary_gchar != NULL)
	{
	    temporary_gchar = NULL;
	    last_row++;
	}
    }
    
    return last_row;
}

/* choose the next or previous race featuring
   a dog from the user's kennel. 'current_race' is the
   race currently displayed, 'type' is like in 'display_races'
   and 'direction' is either 1 for looking upwards in the race
   array or -1 for looking downwards */
gint next_user_race(gint current_race, gint type, gint direction)
{
    gint i,j;
    /* there are only 6 trophy races instead
       of 12 normally */
    gint limit = (type < 40) ? 12 : 6;
    gint new_race;

    if(direction == 1)
	new_race = current_race % limit + 1;
    else
	new_race = (current_race + limit - 2) % limit + 1;
    
    /* the user may have switched this feature off
       to be able to look at all races;
       besides, all trophy races are shown;
       and third, we don't cycle like this when
       entering dogs */
    if(options_misc[2] == 1 || type > 39 || type == 31)
	return new_race;

    if(direction == 1)
	for(i = 0; i < 11; i++)
	    for(j=2;j<4;j++)
	    {
		if(races[0][(new_race - 1 + i) % 12].
		   dogs[j * 3] != NULL &&
		   races[0][(new_race - 1 + i) % 12].
		   dogs[j * 3]->id < 20)
		    return (new_race - 1 + i) % 12 + 1;
	    }
    else
	for(i = 0; i < 12 - 1; i++)
	    for(j=2;j<4;j++)
	    {
		if(races[0][(new_race + 11 - i) % 12].
		   dogs[j * 3] != NULL &&
		   races[0][(new_race + 11 - i) % 12].
		   dogs[j * 3]->id < 20)
		    return (new_race + 11 - i) % 12 + 1;
	    }

    if( (races[0][current_race - 1].dogs[6] != NULL &&
	 races[0][current_race - 1].dogs[6]->id < 20) ||
	(races[0][current_race - 1].dogs[9] != NULL &&
	 races[0][current_race - 1].dogs[9]->id < 20) )
	return current_race;
    else
	return new_race + 100;
}

/* return the next user race including 'current_race' (contrary to the
   above function) the level of which is switched
   on for live races */
gint next_user_race_levels(gint current_race)
{
    gint levels_on[6];
    gint i;

    for(i=5;i<11;i++)
	if(options_misc[i] == 1)
	    levels_on[i - 5] = i - 5;
	else
	    levels_on[i - 5] = -1;
    
    for(i=current_race; i<12;i++)
	if( ((races[0][i].dogs[5] != NULL && races[0][i].dogs[5]->id < 20) ||
	     (races[0][i].dogs[9] != NULL && races[0][i].dogs[9]->id < 20)) &&
	    (is_in_list(levels_on, 0, 5, races[0][i].level) == 1 ||
	     races[0][i].level == -1) )
	    return i;

    return -1;
}

/* find the next kennel with a bitch in heat upwards or
   downwards from current_kennel */
gint next_kennel_in_heat(gint current_kennel, gint direction)
{
    gint i;
    gint new_kennel_id;

    if(direction == 1)
	new_kennel_id = current_kennel % 20 + 1;
    else
	new_kennel_id = (current_kennel + 18) % 20 + 1;

    /* if the user's kennel has a bitch in heat, we return */
    if(kennel_has_bitch_in_heat(0) == 1)
	return new_kennel_id;
    
    if(direction == 1)
    {
	for(i=0;i<19;i++)
	    if(kennel_has_bitch_in_heat((new_kennel_id - 1 + i) % 20) == 1)
		return (new_kennel_id - 1 + i) % 20 + 1;
    }
    else
    {
	for(i=0;i<19;i++)
	    if(kennel_has_bitch_in_heat((new_kennel_id + 19 - i) % 20) == 1)
		return (new_kennel_id + 19 - i) % 20 + 1;
    }
    
    return new_kennel_id;
}

/* pop up a window with help text in it.
   if to_show is 0, the help in the main window has been pressed
   and 'widget' is from the main window.
   if show is -1 or 1, we show the next or previous section 
   depending on the number stored in the help window; in this
   case 'widget' is from the help window.
   if to_show is 50, we show the 'about' text */
gint display_help(GtkWidget *widget, gint to_show)
{
    GtkWidget *help_window;
    GtkWidget *dog_info_label = NULL;
    GtkWidget *dog_info_label_2 = NULL;
    GtkWidget *help_back, *help_forward;
    GtkWidget *help_text_view;
    GtkTextBuffer *help_text_buffer;
    GtkWidget *help_number;
    
    gchar *info_label;
    gchar *info_label_2;
    gint current_number;
    gchar temporary_gchar[120] = "\0";
    gchar trash[20] = "\0";
    gchar section[15] = "\0";
    FILE *fil = fopen(my_find_pixmap_file("help"), "r");

    /* return error if 'help' was not found */
    if(fil == NULL)
	return 1;

    /* we either have to create the window anew */
    if(to_show == 0)
    {
	help_window = create_help_window();
	help_text_view = lookup_widget(help_window, "help_text");
	help_text_buffer =
	    gtk_text_view_get_buffer(GTK_TEXT_VIEW(help_text_view));
	dog_info_label =
	    lookup_widget(widget, "dog_info_label");
	dog_info_label_2 =
	    lookup_widget(widget, "dog_info_label_2");
    }
    else if(to_show == 50)
    {
	help_window = create_help_window();
	help_text_view = lookup_widget(help_window, "help_text");
	help_text_buffer =
	    gtk_text_view_get_buffer(GTK_TEXT_VIEW(help_text_view));
	help_forward = lookup_widget(help_window, "help_forward");
	help_back = lookup_widget(help_window, "help_back");
	
	gtk_widget_set_sensitive(help_forward, FALSE);
	gtk_widget_set_sensitive(help_back, FALSE);
    }
    /* or we have to clear it */
    else
    {
	help_window = lookup_widget(widget, "help_window");
	help_text_view = lookup_widget(help_window, "help_text");
	help_text_buffer =
	    gtk_text_view_get_buffer(GTK_TEXT_VIEW(help_text_view));
	gtk_text_buffer_set_text(help_text_buffer,"", -1);
    }

    help_number = lookup_widget(help_window,
				"help_number");
    current_number = get_int_from_label(GTK_LABEL(help_number))
	+ to_show;
    
    if(current_number == -1)
	current_number = 10;
    else if(current_number == 11)
	current_number = 0;
    
    if(to_show == 0)
    {
	gtk_label_get(GTK_LABEL(dog_info_label), &info_label);
	gtk_label_get(GTK_LABEL(dog_info_label_2), &info_label_2);
    }
    else
    {
	namescan(&info_label, "Dummy");
	namescan(&info_label_2, "Dummy");
    }

    /* find out where the user is */
    if( (current_number == 1 && to_show != 0) || 
	(strncmp(info_label, " Enter", 3) == 0 && to_show == 0) )
    {
	strcpy(section, "entering_dogs");
	gtk_label_set_text(GTK_LABEL(help_number), "1");
    }
    else if( (strncmp(info_label, " Race", 3) == 0 && to_show == 0) ||
	     (current_number == 2 && to_show != 0) )
    {
	strcpy(section, "race_results");
	gtk_label_set_text(GTK_LABEL(help_number), "2");
    }
    else if( (strncmp(info_label, " Last", 3) == 0 && to_show == 0) ||
	     (current_number == 3 && to_show != 0) )
    {
	strcpy(section, "last_races");
	gtk_label_set_text(GTK_LABEL(help_number), "3");
    }
    else if( (strncmp(info_label, " Trad", 3) == 0 && to_show == 0) ||
	     (current_number == 4 && to_show != 0) )
    {
	strcpy(section, "trading_");
	gtk_label_set_text(GTK_LABEL(help_number), "4");
    }
    else if( (strncmp(info_label, " Mati", 3) == 0 && to_show == 0) ||
	     (current_number == 5 && to_show != 0)  )
    {
	strcpy(section, "breeding_");
	gtk_label_set_text(GTK_LABEL(help_number), "5");
    }
    else if( (strncmp(info_label, " Statistics  Dogs", 15 ) == 0 &&
	      to_show == 0) ||
	     (current_number == 6 && to_show != 0)  )
    {
	strcpy(section, "dog_stats");
	gtk_label_set_text(GTK_LABEL(help_number), "6");
    }
    else if( (strncmp(info_label, " Statistics  Kenn", 15 ) == 0 &&
	      to_show == 0) ||
	     (current_number == 7 && to_show != 0)  )
    {
	strcpy(section, "kennel_stats");
	gtk_label_set_text(GTK_LABEL(help_number), "7");
    }
    else if( (strncmp(info_label, " Statistics", 6) == 0 &&
	      strncmp(info_label_2, "Cham", 3) == 0 &&
	      to_show == 0) ||
	     (current_number == 8 && to_show != 0)  )
    {
	strcpy(section, "champions_stats");
	gtk_label_set_text(GTK_LABEL(help_number), "8");
    }
    else if( (strncmp(info_label, " Statistics", 6) == 0 &&
	      strncmp(info_label_2, "All", 3) == 0 &&
	      to_show == 0) ||
	     (current_number == 9 && to_show != 0)  )
    {
	strcpy(section, "all_time_stats");
	gtk_label_set_text(GTK_LABEL(help_number), "9");
    }
    else if( (strncmp(info_label, " Statistics", 6) == 0 &&
	      strncmp(info_label_2, "His", 3) == 0 &&
	      to_show == 0) ||
	     (current_number == 10 && to_show != 0)  )
    {
	strcpy(section, "history_stats");
	gtk_label_set_text(GTK_LABEL(help_number), "10");
    }
    else if(to_show == 50)
	strcpy(section, "about_");
    else
    {
	strcpy(section, "main_screen");
	gtk_label_set_text(GTK_LABEL(help_number), "0");
    }

    /* scan forward until the right section */
    while(strcmp(temporary_gchar, section) != 0)
    {
	fscanf(fil,"%[^\n]", temporary_gchar);
	fscanf(fil,"%[\n]", trash);
    }

    /* remove the section name */
    fscanf(fil,"%[^\n]", temporary_gchar);

    /* write text to help screen until section end */
    while(strcmp(temporary_gchar, section) != 0)
    {
	gtk_text_buffer_insert_at_cursor(help_text_buffer,
					 temporary_gchar, -1);
	fscanf(fil,"%[\n]*", trash);
	gtk_text_buffer_insert_at_cursor(help_text_buffer,
					 trash, -1);
	fscanf(fil,"%[^\n]", temporary_gchar);
    }

    /* show help window */
    if(to_show == 0 || to_show == 50)
	gtk_widget_show(help_window);

    return 0;
}

gint write_labels(GtkWidget *widget)
{
    GtkWidget *selected_row_dog_list =
	lookup_widget(widget, "selected_row_dog_list");
    GtkWidget *selected_row_dog_info =
	lookup_widget(widget, "selected_row_dog_info");
    GtkWidget *season_number = lookup_widget(widget,
					     "season_number");
    GtkWidget *month_number = lookup_widget(widget,
					    "month_number");
    GtkWidget *week_number = lookup_widget(widget,
					   "week_number");
    gchar number[4];
    
    gtk_label_set(GTK_LABEL(selected_row_dog_info), "0");
    gtk_label_set(GTK_LABEL(selected_row_dog_list), "0");

    label_set_text_from_int(GTK_LABEL(season_number), season, 0);
    label_set_text_from_int(GTK_LABEL(month_number), month, 0);
    label_set_text_from_int(GTK_LABEL(week_number), week, 0);

    return 0;
}

/* set the label that indicates whether the current game is saved
   or not to 1 (saved) or 0 (not).
   if 'status' is -1 we simply return the current status */
gint set_save_status(GtkWidget *widget, gint status)
{
    GtkWidget *save_status = lookup_widget(widget,
					   "save_status");
    int current_status = get_int_from_label(GTK_LABEL(save_status));

    if(status == -1)
	return current_status;
    else
	label_set_text_from_int(GTK_LABEL(save_status), status, 0);
    
    return 0;
}

/* change the 'popups_active' label */
gint change_popups_active(GtkWidget *widget, gint difference)
{
    GtkWidget *main_window = lookup_widget(widget,
					   "main_window");
    GtkWidget *popups_active = lookup_widget(widget, "popups_active");
    
    gint pop_number = get_int_from_label(GTK_LABEL(popups_active)) + 
	difference;

    label_set_text_from_int(GTK_LABEL(popups_active), pop_number, 0);

    if(pop_number == 0)
	gtk_widget_set_sensitive(main_window, TRUE);
    else
	gtk_widget_set_sensitive(main_window, FALSE);

    return 0;
}

/* show a 'live race': a couple of progress bars racing against each
   other */
gint show_live_race(GtkWidget *live_race_window, gint race_number)
{
    GtkWidget *progress_bars[10];
    GtkWidget *labels[11];
    GtkWidget *buttons[3];
    race Race = races[0][race_number];
    gfloat fractions[10][500];
    gint steps[10];
    gint i,j;
    gint last_dog = (Race.level == -1) ? 6 : 10;
    gint last_rank = 0;
    gint injury_time[last_dog];
    gint winning_color = 22;

    if(live_race_window == NULL)
    {
	live_race_window = create_live_race_window();
	gtk_widget_show(live_race_window);
	change_popups_active(main_window, 1);
    }
    
    get_live_race_pointers(live_race_window, progress_bars,
			   NULL, labels, buttons);

    for(i=last_dog;i<10;i++)
	gtk_widget_hide(progress_bars[i]->parent);

    gtk_widget_hide(buttons[2]);

    if(next_user_race_levels(race_number + 1) == -1)
	gtk_widget_set_sensitive(buttons[0], FALSE);
       
    /* highlight human player's dog */
    if(Race.level == -1)
	set_color_progress_bar(progress_bars[5], 30);
    else
	set_color_progress_bar(progress_bars[9], 30);
    
    set_live_race_labels(live_race_window, race_number);
   
    /* find out the last dog with a normal time (not injured) */
    for(i=0;i<last_dog;i++)
    {
	injury_time[i] = 500;
	if(Race.times[Race.ranks[i]] < 100)
	    last_rank = i + 1;
    }

    /* assign random injury times that will determine when
       the corresponding progress bar gets frozen */
    if(last_rank != last_dog)
	injury_time[Race.ranks[last_rank]] = (gint)rint( rnd(300, 450) );
    for(i = last_rank + 1; i < last_dog; i++)
	injury_time[Race.ranks[i]] =
	    (gint)rint( rnd(50, injury_time[Race.ranks[i - 1]]) );

    for(i=0;i<last_dog;i++)
    {
	if(i < last_rank)
	    steps[Race.ranks[i]] = (gint)rint(
		100 / (Race.times[Race.ranks[last_rank - 1]] -
		       Race.times[Race.ranks[0]])
		* (Race.times[Race.ranks[i]] -
		   Race.times[Race.ranks[0]]) + 400 );
	else
	    steps[Race.ranks[i]] =
		(int)rint( rnd(steps[Race.ranks[0]],
			       steps[Race.ranks[last_rank - 1]]) );
	
	generate_update_fractions(fractions[Race.ranks[i]],
				  steps[Race.ranks[i]]);
    }

    for(i=0;i<500;i++)
    {
	for(j=0;j<last_dog;j++)
	{
	    if(j < last_rank ||
	       (j >= last_rank && i < injury_time[Race.ranks[j]]) )
		gtk_progress_bar_set_fraction(
		    GTK_PROGRESS_BAR(progress_bars[Race.ranks[j]]),
		    fractions[Race.ranks[j]][i]);
	    
	    /* indicate injury with a color change */
	    if(i == injury_time[Race.ranks[j]])
	    {
		label_set_text(GTK_LABEL(labels[Race.ranks[j]]),
			       "(injury) ", 0);
		label_set_text_from_int(
		    GTK_LABEL(labels[Race.ranks[j]]), j + 1, 1);
		label_set_text(GTK_LABEL(labels[Race.ranks[j]]), ".", 1);
	    }
	    if(i == injury_time[Race.ranks[j]] + 1)
		set_color_progress_bar(progress_bars[Race.ranks[j]], 21);

	    /* write rank label */
	    if(i == steps[Race.ranks[j]] - 1 &&
	       j < last_rank)
	    {
		if(Race.times[Race.ranks[j]] == 
		   dog_records[Race.laps - 1].value)
		{
		    winning_color = 10;
		    label_set_text(GTK_LABEL(labels[Race.ranks[j]]),
				   "(NR) ", 0);
		}
		
		label_set_text_from_int(
		    GTK_LABEL(labels[Race.ranks[j]]), j + 1, 1);
		label_set_text(GTK_LABEL(labels[Race.ranks[j]]), ".", 1);
	    }
	
	    if(i == steps[Race.ranks[0]])
		set_color_progress_bar(progress_bars[Race.ranks[0]],
				       winning_color);
	}
		
	/* the label is 1 if the user's clicked on 'cancel' */
	if(strcmp(gtk_label_get_text(GTK_LABEL(labels[10])), "-1") == 0)
	{
	    gtk_widget_destroy(live_race_window);
	    change_popups_active(main_window, -1);
	    return 1;
	}

	while(gtk_events_pending())
	    gtk_main_iteration();
	
	usleep(1000 + options_misc[0] * 1000);
    }

    label_set_text_from_int(GTK_LABEL(labels[10]), -1, 0);

    gtk_widget_hide(buttons[1]);
    gtk_widget_show(buttons[2]);    

    return 0;
}

/* generate a growing sequence of fractions from 0 to 1 which
   give the race progress of a dog; the smaller max_idx the faster
   the race time */
gint generate_update_fractions(gfloat *update_fractions, gint max_idx)
{
    gint i;
    gfloat delta_fractions[500];
    gfloat sum = 0;

    delta_fractions[0] = (gint)rint( rnd(2,4) );
    
    for(i=1;i<max_idx;i++)
    {
	delta_fractions[i] = rnd(delta_fractions[i - 1] - 0.15,
				 delta_fractions[i - 1] + 0.15);

	if(delta_fractions[i] < 0.15)
	    delta_fractions[i] = 0.15;
	else if(delta_fractions[i] > 4)
	    delta_fractions[i] = 4;

	sum += delta_fractions[i];
    }

    for(i=max_idx;i<500;i++)
	delta_fractions[i] = 0;
    
    for(i=0;i<500;i++)
    {
	delta_fractions[i] *= 1 / sum;
	if(i == 0)
	    update_fractions[i] = delta_fractions[i];
	else
	    update_fractions[i] = update_fractions[i - 1] + 
		delta_fractions[i];

	if(update_fractions[i] > 1)
	    update_fractions[i] = 1;
    }

    return 0;
}

/* set some race data and the dog names in the live race window */
gint set_live_race_labels(GtkWidget *live_race_window,
			  gint race_nr)
{
    GtkWidget *dog_names[10];
    GtkWidget *dog_labels[10];
    GtkWidget *race_level = lookup_widget(live_race_window,
					  "race_level");
    GtkWidget *race_laps = lookup_widget(live_race_window,
					 "race_laps");
    GtkWidget *race_number = lookup_widget(live_race_window,
					   "race_number");
    gint i;

    if(races[0][race_nr].level != -1)
	label_set_text_from_int(GTK_LABEL(race_level),
				races[0][race_nr].level, 0);
    else
	gtk_label_set_text(GTK_LABEL(race_level),
			   "any");
    
    label_set_text_from_int(GTK_LABEL(race_laps),
			    races[0][race_nr].laps, 0);
    label_set_text_from_int(GTK_LABEL(race_number), race_nr + 1, 0);

    get_live_race_pointers(live_race_window,NULL,
			   dog_names, dog_labels, NULL);

    for(i=0;i<10;i++)
    {
	if(races[0][race_nr].dogs[i] != NULL)
	    gtk_label_set_text(GTK_LABEL(dog_names[i]),
			       races[0][race_nr].dogs[i]->name);
	
	gtk_label_set_text(GTK_LABEL(dog_labels[i]), "");
    }

    return 0;
}

/* set or append 'number' as text into 'label' */
gint label_set_text_from_int(GtkLabel *label, gint number, gint append)
{
    const gchar *current_text = gtk_label_get_text(label);
    gchar temporary_char[500] = "\0";
    
    if(append == 0)
	sprintf(temporary_char, "%d", number);
    else
	sprintf(temporary_char, "%s%d", current_text, number);

    gtk_label_set_text(label, temporary_char);

    return 0;
}

/* set or append 'text' into 'label' */
gint label_set_text(GtkLabel *label, gchar *text, gint append)
{
    const gchar *current_text = gtk_label_get_text(label);
    gchar temporary_char[500] = "\0";

    if(append == 0)
	strcpy(temporary_char, text);
    else
	sprintf(temporary_char, "%s%s", current_text, text);

    gtk_label_set_text(label, temporary_char);
    
    return 0;
}

/* change the color of a progress bar */
gint set_color_progress_bar(GtkWidget *progress_bar, gint type)
{
    GdkColor color;
    
    set_color(&color, type);

    gtk_widget_modify_bg(progress_bar, GTK_STATE_PRELIGHT, &color);

    return 0;    
}

/* write pointers to the respective widgets in the live race window
   into the arguments */
gint get_live_race_pointers(GtkWidget *live_race_window,
			    GtkWidget **progress_bars, GtkWidget **dog_names,
			    GtkWidget **labels, GtkWidget **buttons)
{
    if(progress_bars != NULL)
    {
	progress_bars[0] = lookup_widget(live_race_window, "progressbar1");
	progress_bars[1] = lookup_widget(live_race_window, "progressbar2");
	progress_bars[2] = lookup_widget(live_race_window, "progressbar3");
	progress_bars[3] = lookup_widget(live_race_window, "progressbar4");
	progress_bars[4] = lookup_widget(live_race_window, "progressbar5");
	progress_bars[5] = lookup_widget(live_race_window, "progressbar6");
	progress_bars[6] = lookup_widget(live_race_window, "progressbar7");
	progress_bars[7] = lookup_widget(live_race_window, "progressbar8");
	progress_bars[8] = lookup_widget(live_race_window, "progressbar9");
	progress_bars[9] = lookup_widget(live_race_window, "progressbar10");
    }
    
    if(dog_names != NULL)
    {
	dog_names[0] = lookup_widget(live_race_window, "dog_1");
	dog_names[1] = lookup_widget(live_race_window, "dog_2");
	dog_names[2] = lookup_widget(live_race_window, "dog_3");
	dog_names[3] = lookup_widget(live_race_window, "dog_4");
	dog_names[4] = lookup_widget(live_race_window, "dog_5");
	dog_names[5] = lookup_widget(live_race_window, "dog_6");
	dog_names[6] = lookup_widget(live_race_window, "dog_7");
	dog_names[7] = lookup_widget(live_race_window, "dog_8");
	dog_names[8] = lookup_widget(live_race_window, "dog_9");
	dog_names[9] = lookup_widget(live_race_window, "dog_10");
    }

    if(labels != NULL)
    {
	labels[0] = lookup_widget(live_race_window, "dog_1_label");
	labels[1] = lookup_widget(live_race_window, "dog_2_label");
	labels[2] = lookup_widget(live_race_window, "dog_3_label");
	labels[3] = lookup_widget(live_race_window, "dog_4_label");
	labels[4] = lookup_widget(live_race_window, "dog_5_label");
	labels[5] = lookup_widget(live_race_window, "dog_6_label");
	labels[6] = lookup_widget(live_race_window, "dog_7_label");
	labels[7] = lookup_widget(live_race_window, "dog_8_label");
	labels[8] = lookup_widget(live_race_window, "dog_9_label");
	labels[9] = lookup_widget(live_race_window, "dog_10_label");
	
	labels[10] = lookup_widget(live_race_window, "live_race_label");
    }

    if(buttons != NULL)	
    {
	buttons[0] = lookup_widget(live_race_window, "live_race_next");
	buttons[1] = lookup_widget(live_race_window, "live_race_cancel");
	buttons[2] = lookup_widget(live_race_window, "live_race_close");
    }

    return 0;
}
